var KeystoreController = Class.create({

    view: null,
    config: null,

    _constructor: function(config) {
        this.view = new KeystoreView();
        this.config = Object.extend(Object.extend({}, KeystoreController.DEFAULT_CONFIG), config);
    },

    start: function() {
        this.view.show({
            onGenerateKeyPair: this._handleGeneratePublicKey.bind(this),
            onImportPublicKey: this._handleImportPublicKey.bind(this),
            onDeletePublicKey: this._handleDeletePublicKey.bind(this),

            onGeneratePgpKey: this._handleGeneratePpgKey.bind(this),
            onImportPgpKey: this._handleImportPgpKey.bind(this),
            onDeletePgpKey: this._handleDeletePgpKey.bind(this)
        });
    },

    _handleGeneratePublicKey: function(view) {
        view.hideMessage();

        var dialog = new GenerateKeyDialog("GenerateSSHKeyDialog", this.config.generateKeyPairUrl);
        dialog.setAlgorithms(this.config.algorithms);
        dialog.setSize(480, 280);
        dialog.show({
            onDone: this._handleDoneGenerateKey.bind(this)
        }, this.config.metadata);
    },

    _handleGeneratePpgKey: function(view) {
        view.hideMessage();

        var dialog = new GenerateKeyDialog("GeneratePgpKeyDialog", this.config.generatePgpKeyUrl);
        dialog.setAlgorithms(this.config.algorithms);
        dialog.setSize(480, 326);
        dialog.show({
            onDone: this._handleDoneGenerateKey.bind(this)
        }, this.config.metadata);
    },

    _handleImportPublicKey: function(view) {
        view.hideMessage();

        var dialog = new ImportKeyDialog("ImportSSHKeyDialog", this.config.importPublicKeyUrl);
        dialog.show({
            onDone: this._handleDoneImportKey.bind(this)
        }, this.config.metadata);
    },

    _handleImportPgpKey: function(view) {
        view.hideMessage();

        var dialog = new ImportKeyDialog("ImportPGPKeyDialog", this.config.importPgpKeyUrl);
        dialog.show({
            onDone: this._handleDoneImportKey.bind(this)
        }, this.config.metadata);
    },

    _handleDeletePublicKey: function(view) {
        var channel = new IFrameChannel();
        channel.open(this.config.deletePublicKeyUrl, "post", {
            onComplete: this._handleCompleteDeleteKey.bind(this),
            onError: function(ch) {ch.close();},
            onTimeout: function(ch) {ch.close();}
        });
        channel.send(this.config.metadata);
    },

    _handleDeletePgpKey: function(view) {
        var channel = new IFrameChannel();
        channel.open(this.config.deletePgpKeyUrl, "post", {
            onComplete: this._handleCompleteDeleteKey.bind(this),
            onError: function(ch) {ch.close();},
            onTimeout: function(ch) {ch.close();}
        });
        channel.send(this.config.metadata);
    },

    _handleCompleteDeleteKey: function(channel) {
        var metadata = channel.getResponseMetadata();
        channel.close();
        var state = metadata["x-dialog-control"];
        var reason = metadata["x-dialog-reason"];
        if ("error" == state) {
            this.view.showError(reason);
        } else if ("done" == state) {
            this.view.showMessage(reason);
        }
    },

    _handleDoneGenerateKey: function(dialog) {
        var message = dialog.getDoneMessage();
        var url = dialog.getDownloadUrl();
        dialog.dispose();

        this.view.showMessage(message);
        this._downloadGeneratedKey(url, this.config.metadata);
    },

    _handleDoneImportKey: function(dialog) {
        var message = dialog.getDoneMessage();
        dialog.dispose();
        this.view.showMessage(message);
    },

    _downloadGeneratedKey: function(url, data) {
        var channel = new IFrameChannel();
        var token = "keystore-" + (new Date().getTime()).toString(36);
        channel.setLoadCompleteToken(token);
        channel.open(url, "post", Object.extend(new Channel.Listener(), {
            onError: function(ch) {ch.close();},
            onTimeout: function(ch) {ch.close();}
        }));
        channel.send(data);
    }
});
KeystoreController.DEFAULT_CONFIG = {
    generateKeyPairUrl: "",
    importPublicKeyUrl: "",
    deletePublicKeyUrl: "",
    generatePgpKeyUrl: "",
    importPgpKeyUrl: "",
    deletePgpKeyUrl: "",
    algorithms: [],
    metadata: {}
};

var KeystoreView = Class.create({

    messageBar: null,

    generateSshKeyButton: null,
    generateKeyPairHandler: null,
    importSshKeyButton: null,
    importPublicKeyHandler: null,
    deletePublicKeyButton: null,
    deletePublicKeyHandler: null,

    generatePgpKeyButton: null,
    generatePgpKeyHandler: null,
    importPgpKeyButton: null,
    importPgpKeyHandler: null,
    deletePgpKeyButton: null,
    deletePgpKeyHandler: null,

    listener: null,

    _constructor: function() {
        this.messageBar = $("messagebar");
        this.generateSshKeyButton = $("gensshkey");
        this.generatePgpKeyButton = $("genpgpkey");
        this.importSshKeyButton = $("importsshkey");
        this.importPgpKeyButton = $("importpgpkey");
        this.deletePublicKeyButton = $("delsshpubkey");
        this.deletePgpKeyButton = $("delpgpkey");
    },

    show: function(listener) {
        this.listener = Object.extend(new KeystoreView.Listener(), listener);
        this.bindEvents();
    },

    hide: function() {
        this.unbindEvents();
        this.listener = null;
    },

    showMessage: function(message) {
        if (Object.isString(message)) {
            this._showMessage(this._createSuccessMessage(message));
        }
    },

    showError: function(message) {
        if (Object.isString(message)) {
            this._showMessage(this._createErrorMessage(message));
        }
    },

    hideMessage: function() {
        while (this.messageBar.hasChildNodes()) {
            this.messageBar.removeChild(this.messageBar.lastChild);
        }
    },

    bindEvents: function() {
        this.generateKeyPairHandler = this._bindHandlerFor(this.generateSshKeyButton, this._handleGenerateKeyPair);
        this.generatePgpKeyHandler = this._bindHandlerFor(this.generatePgpKeyButton, this._handleGeneratePgpKey);
        this.importPublicKeyHandler = this._bindHandlerFor(this.importSshKeyButton, this._handleImportPublicKey);
        this.importPgpKeyHandler = this._bindHandlerFor(this.importPgpKeyButton, this._handleImportPgpKey);
        this.deletePublicKeyHandler = this._bindHandlerFor(this.deletePublicKeyButton, this._handleDeletePublicKey);
        this.deletePgpKeyHandler = this._bindHandlerFor(this.deletePgpKeyButton, this._handleDeletePgpKey);
    },

    unbindEvents: function() {
        this._unbindHandlerFor(this.generateSshKeyButton, this.generateKeyPairHandler);
        this._unbindHandlerFor(this.generatePgpKeyButton, this.generatePgpKeyHandler);
        this._unbindHandlerFor(this.importSshKeyButton, this.importPublicKeyHandler);
        this._unbindHandlerFor(this.importPgpKeyButton, this.importPgpKeyHandler);
        this._unbindHandlerFor(this.deletePublicKeyButton, this.deletePublicKeyHandler);
        this._unbindHandlerFor(this.deletePgpKeyButton, this.deletePgpKeyHandler);
        this.generateKeyPairHandler = null;
        this.generatePgpKeyHandler = null;
        this.importPublicKeyHandler = null;
        this.importPgpKeyHandler = null;
        this.deletePublicKeyHandler = null;
        this.deletePgpKeyHandler = null;
    },

    _bindHandlerFor: function(button, handler) {
        if (button) {
            var f = handler.bindAsEventListener(this);
            Event.observe(button, "click", f);
            return f;
        }
        return null;
    },

    _unbindHandlerFor: function(button, handler) {
        if (button && handler) {
            Event.unobserve(button, "click", handler);
        }
    },

    _handleGenerateKeyPair: function(e) {
        Event.stop(e);
        this.listener.onGenerateKeyPair(this);
    },

    _handleGeneratePgpKey: function(e) {
        Event.stop(e);
        this.listener.onGeneratePgpKey(this);
    },

    _handleImportPublicKey: function(e) {
        Event.stop(e);
        this.listener.onImportPublicKey(this);
    },

    _handleImportPgpKey: function(e) {
        Event.stop(e);
        this.listener.onImportPgpKey(this);
    },

    _handleDeletePublicKey: function(e) {
        Event.stop(e);
        this.listener.onDeletePublicKey(this);
    },

    _handleDeletePgpKey: function(e) {
        Event.stop(e);
        this.listener.onDeletePgpKey(this);
    },

    _createSuccessMessage: function(text) {
        var title = document.createElement("H3");
        title.appendChild(document.createTextNode(text));
        var result = document.createElement("DIV");
        Utils.addCssClass(result, "alertbox");
        result.appendChild(title);
        return result;
    },

    _createErrorMessage: function(text) {
        var result = document.createElement("DIV");
        Utils.addCssClass(result, "alertbox ");
        Utils.addCssClass(result, "error");
        var title = document.createElement("H3");
        title.appendChild(document.createTextNode(KeystoreView.i18n.ERROR_TITLE));
        var message = document.createElement("P");
        message.appendChild(document.createTextNode(text));
        result.appendChild(title);
        result.appendChild(message);
        return result;
    },

    _showMessage: function(html) {
        this.hideMessage();
        this.messageBar.appendChild(html);
    }
});
KeystoreView.i18n = {ERROR_TITLE: "A problem occurred with your request:"};
KeystoreView.Listener = Class.create({
    onGenerateKeyPair: function(view) {},
    onImportPublicKey: function(view) {},
    onDeletePublicKey: function(view) {},
    onGeneratePgpKey: function(view) {},
    onImportPgpKey: function(view) {},
    onDeletePgpKey: function(view) {}
});

var GenerateKeyDialog = Class.create({

    dialog: null,
    algorithms: [],
    keyTypeBox: null,
    keyLengthBox: null,
    listener: null,

    _constructor: function(id, formDataUrl) {
        this.dialog = new FormDialog(id, formDataUrl);
    },

    isDone: function() {
        return this.dialog.isDone();
    },

    getDoneMessage: function() {
        return this.dialog.getDoneMessage();
    },

    getDownloadUrl: function() {
        return this.dialog.getMetaValue("x-dialog-download-url");
    },

    setAlgorithms: function(values) {
        this.algorithms = values || [];
    },

    setSize: function(width, height) {
        this.dialog.setSize(width, height);
    },

    show: function(listener, data) {
        this.listener = listener || GenerateKeyDialog.Listener;

        this.dialog.show(Object.extend(new FormDialog.Listener(), {
            onLoaded: this._handleLoaded.bind(this),
            onLoadError: this._handleLoadError.bind(this),
            onSubmit: this._handleSubmit.bind(this),
            onCancel: this._handleCancel.bind(this),
            onDone: this._handleDone.bind(this)
        }), data);
    },

    hide: function() {
        this.dialog.hide();
        this.listener = null;
    },

    dispose: function() {
        this.dialog.dispose();
        this.listener = null;
    },

    _handleLoaded: function() {
        this._initComponents();
        this._initListeners();
    },

    _handleLoadError: function() {
        this._setSubmitButtonText(FormDialog.Buttons.OK);
    },

    _handleSubmit: function() {
        this._setSubmitButtonText(GenerateKeyDialog.Messages.GENERATING);
    },

    _handleCancel: function() {
        this.dispose();
    },

    _handleDone: function() {
        this.listener.onDone(this);
    },

    _initComponents: function() {
        this.keyTypeBox = $("keytype");
        this.keyLengthBox = $("keylength");
        this._setCancelButtonText(GenerateKeyDialog.Buttons.CANCEL);
        this._setSubmitButtonText(GenerateKeyDialog.Buttons.OK);
        this._setOptions(this.keyTypeBox, this._keyTypes());
        this._onChange$KeyType();
    },

    _initListeners: function() {
        Event.observe(this.keyTypeBox, "change", this._onChange$KeyType.bind(this));
    },

    _onChange$KeyType: function() {
        var lengths = this._lengthsFor($F(this.keyTypeBox));
        this._setOptions(this.keyLengthBox, lengths);
    },

    _keyTypes: function() {
        var types = [];
        for (var i = 0; i < this.algorithms.length; i++) {
            types.push(this.algorithms[i].name);
        }
        return types;
    },

    _lengthsFor: function(name) {
        for (var i = 0; i < this.algorithms.length; i++) {
            var algorithm = this.algorithms[i];
            if (algorithm.name == name) {
                return algorithm.lengths || [];
            }
        }
        return [];
    },

    _setOptions: function(select, options) {
        while (select.options.length > 0) {
            select.removeChild(select.options[0]);
        }
        for (var i = 0; i < options.length; i++) {
            select.appendChild(Utils.createOption(options[i]));
        }
    },

    _setKeyLength: function(values) {
        while (this.keyLengthBox.options.length != null) {
            this.keyLengthBox.removeChild(this.keyLengthBox.options[0]);
        }
        for (var i = 0; i < values.length; i++) {
            this.keyLengthBox.appendChild(Utils.createOption(values[i]));
        }
    },

    _setSubmitButtonText: function(value) {
        this._setButtonText(ModalDialog.BUTTON_OK, value || FormDialog.Buttons.OK);
    },

    _setCancelButtonText: function(value) {
        this._setButtonText(ModalDialog.BUTTON_CANCEL, value || FormDialog.Buttons.CANCEL);
    },

    _setButtonText: function(type, text) {
        this.dialog.getButton(type).value = text;
    }
});

GenerateKeyDialog.Buttons = {OK: 'OK', CANCEL: 'Cancel'};
GenerateKeyDialog.Messages = {GENERATING: "Generating..."};
GenerateKeyDialog.Listener = {
    onDone: function(dialog) {}
};

var ImportKeyDialog = Class.create({

    dialog: null,
    listener: null,

    _constructor: function(id, formDataUrl) {
        this.dialog = new FormDialog(id, formDataUrl);
    },

    isDone: function() {
        return this.dialog.isDone();
    },

    getDoneMessage: function() {
        return this.dialog.getDoneMessage();
    },

    show: function(listener, data) {
        this.listener = listener || ImportKeyDialog.LISTENER;
        this.dialog.setSize(520, 240);
        this.dialog.show(Object.extend(new FormDialog.Listener(), {
            onLoaded: this._handleLoaded.bind(this),
            onLoadError: this._handleLoadError.bind(this),
            onSubmit: this._handleSubmit.bind(this),
            onCancel: this._handleCancel.bind(this),
            onDone: this._handleDone.bind(this)
        }), data);
    },

    hide: function() {
        this.dialog.hide();
        this.listener = null;
    },

    dispose: function() {
        this.dialog.dispose();
        this.listener = null;
    },

    _handleLoaded: function() {
        this._setSubmitButtonText(ImportKeyDialog.Buttons.OK);
        this._setCancelButtonText(ImportKeyDialog.Buttons.CANCEL);
    },

    _handleLoadError: function() {
        this._setSubmitButtonText(FormDialog.Buttons.OK);
    },

    _handleSubmit: function() {
        this._setSubmitButtonText(ImportKeyDialog.Messages.IMPORTING);
    },

    _handleCancel: function() {
        this.dispose();
    },

    _handleDone: function() {
        this.listener.onDone(this);
    },

    _setSubmitButtonText: function(value) {
        this._setButtonText(ModalDialog.BUTTON_OK, value || FormDialog.Buttons.OK);
    },

    _setCancelButtonText: function(value) {
        this._setButtonText(ModalDialog.BUTTON_CANCEL, value || FormDialog.Buttons.CANCEL);
    },

    _setButtonText: function(type, text) {
        this.dialog.getButton(type).value = text;
    }
});

ImportKeyDialog.Buttons = {OK: 'OK', CANCEL: 'Cancel'};
ImportKeyDialog.Messages = {IMPORTING: "Importing..."};
ImportKeyDialog.LISTENER = {
    onDone: function(dialog) {}
};