var FileListView = Class.create({

    items: null,
    markedItems: null,
    markedItemsBackup: null,

    selectAllBox: null,
    selectAllHandler: null,

    listener: null,

    _constructor: function(fileTable, selectAllBox) {
        this.items = [];
        this.markedItems = [];
        this.markedItemsBackup = [];

        this.selectAllBox = selectAllBox;

        this._init(fileTable);
    },

    getItems: function() {
        return this.items;
    },

    getSelectedItems: function() {
        return this.markedItems.clone();
    },

    show: function(listener) {
        this.listener = listener || new FileListView.Listener();
        this._bindEvents();
    },

    hide: function() {
        this._unbindEvents();
        this.listener = null;
    },

    destroy: function() {
        while (this.items.length != 0) {
            this.items.pop().destroy();
        }

        this.markedItems = null;
        this.markedItemsBackup = null;
        this.selectAllHandler = null;
    },

    _init: function(table) {
        this.selectAllHandler = this._handleSelectAll.bind(this);
        var listener  = {
            onMove: this._handleMove.bind(this),
            onMoveStart: this._handleMoveStart.bind(this),
            onMoveCancel: this._handleMoveCancel.bind(this),
            onGet: this._handleGetItem.bind(this),
            onSelect: this._handleSelection.bind(this),
            onMark: this._handleMarkup.bind(this)
        };

        var rows = table.tBodies[0].rows;
        for (var i = 0, size = rows.length; i < size; ++i) {
            var item = FileListView.Item.fromRow(rows[i], listener);
            this.items.push(item);
        }
    },

    _handleGetItem: function(item) {
        if (item.access.change) {
            this.listener.onChangeDir(item.path);
        } else if (item.access.dload) {
            this.listener.onDloadFile(item.path);
        }
    },

    _handleSelectAll: function() {
        var selected = this.selectAllBox.checked;
        this.markedItems = [];

        for (var i = 0; i < this.items.length; ++i) {
            var item = this.items[i];
            item.setMarked(selected);
            if (item.isMarked()) {
                this.markedItems.push(item);
            }
            if (item.isMarkable()) {
                item.setSelected(selected);
            }
        }

        this._updateSelectAll();
        this.listener.onSelectionChanged(this);
    },

    _handleSelection: function(item, selected) {
        item.setSelected(selected);

        if (item.isMarked() != selected) {
            item.setMarked(selected);
            this._handleMarkup(item);
        }

        this.listener.onSelectionChanged(this);
    },

    _handleMarkup: function(item) {
        var index = this.markedItems.indexOf(item);
        item.setSelected(item.isMarked());

        if (index == -1 && item.isMarked()) {
            this.markedItems.push(item);
            this.listener.onSelectionChanged(this);
        } else if (index != -1 && !item.isMarked()) {
            this.markedItems.splice(index, 1);
            this.listener.onSelectionChanged(this);
        }

        this._updateSelectAll();
    },

    _handleMoveStart: function(item) {
        if (this.markedItems.contains(item)) {
            for (var i = 0; i < this.markedItems.length; ++i) {
                this._markAsMoving(this.markedItems[i], true);
            }
        } else {
            this._backupSelection();
            this.markedItems.push(item);
        }

        this._markAsMoving(item, true);
    },

    _handleMove: function(source, target) {
        var sources = [source];
        for (var i = 0; i < this.markedItems.length; ++i) {
            var item = this.markedItems[i];
            this._markAsMoving(item, false);
            if (item.path != source) {
                sources.push(item.path);
            }
        }

        this._restoreSelection();
        this.listener.onMoveFile(sources, target);
    },

    _handleMoveCancel: function(item) {
        this._markAsMoving(item, false);
        for (var i = 0; i < this.markedItems.length; ++i) {
            this._markAsMoving(this.markedItems[i], false);
        }

        this._restoreSelection();
    },

    _bindEvents: function() {
        Event.observe(this.selectAllBox, "click", this.selectAllHandler);

        for (var i = 0, size = this.items.length; i < size; ++i) {
            var item = this.items[i];
            item._bindEvents();
        }
    },

    _unbindEvents: function() {
        Event.unobserve(this.selectAllBox, "click", this.selectAllHandler);

        for (var i = 0, size = this.items.length; i < size; ++i) {
            var item = this.items[i];
            item._unbindEvents();
        }
    },

    _backupSelection: function() {
        this.markedItemsBackup = [];
        while (this.markedItems.length != 0) {
            var entry = this.markedItems.pop();
            entry.setMarked(false);
            entry.setSelected(false);
            this.markedItemsBackup.push(entry);
        }
        this.markedItems = [];
    },

    _restoreSelection: function() {
        if (this.markedItemsBackup.length == 0) {
            return;
        }

        this.markedItems = [];
        while (this.markedItemsBackup.length != 0) {
            var entry = this.markedItemsBackup.pop();
            entry.setMarked(true);
            entry.setSelected(true);
            this.markedItems.push(entry);
        }
    },

    _updateSelectAll: function() {
        var wholeSelection = true;

        if (this.markedItems.length == 0) {
            this.selectAllBox.checked = false;
        } else {
            this.selectAllBox.checked = true;

            for (var i = 0; i < this.items.length; ++i) {
                var item = this.items[i];
                if (item.isMarkable() && !item.isMarked()) {
                    wholeSelection = false;
                    break;
                }
            }
        }

        Utils.setStyle(this.selectAllBox, {opacity: wholeSelection ? 1 : .5});
    },

    _markAsMoving: function(item, moving) {
        item.setSelected(moving);
        item.setMarked(moving);
        Utils.setStyle(item.node, {opacity: moving ? .4 :  1});
    }

});

FileListView.Item = Class.create({

    row: null,
    node: null,
    markBox: null,

    path: null,
    access: null,
    selected: false,
    dndHandles: null,
    mouseHandler: null,
    keyboardHandler: null,

    listener: null,

    _constructor: function(row, node, markBox, listener) {
        this.row = row;
        this.node = node;
        this.markBox = markBox;
        this.path = this.node.getAttribute("data-path") ||  (Utils.getInnerText(this.node) || "");
        this.access = new FileListView.Access(this.node.getAttribute("data-access"));
        this.listener = listener;

        this.dndHandlers = {
            start: this._handleDragStart.bindAsEventListener(this),
            end: this._handleDragEnd.bind(this),
            enter: this._handleDragEnter.bindAsEventListener(this),
            over: this._handleDragOver.bindAsEventListener(this),
            leave: this._handleDragLeave.bind(this),
            drop: this._handleDrop.bindAsEventListener(this)
        };
        this.mouseHandler = this._handleMouseEvent.bindAsEventListener(this);
        this.keyboardHandler = this._handleKeyboardEvent.bindAsEventListener(this);
    },

    destroy: function() {
        this._unbindEvents();
        this.row = null;
        this.node = null;
        this.dndHandles = null;
        this.mouseHandler = null;
        this.listener = null;
    },

    update: function() {
        if (this.selected) {
            Utils.addCssClass(this.row, "sel");
        } else {
            Utils.removeCssClass(this.row, "sel");
        }
    },

    isMarkable: function() {
        return this.markBox != null;
    },

    isMarked: function() {
        return this.markBox && this.markBox.checked;
    },

    setMarked: function(value) {
        if (this.markBox) {
            this.markBox.checked = value;
        }
    },

    setSelected: function(value) {
        this.selected = value;
        this.update();
    },

    _bindEvents: function() {
        this._bindMouseEvent();
        this._bindDragDropEvents();
    },

    _bindMouseEvent: function() {
        Event.observe(this.row, "click", this.mouseHandler);
    },

    _bindDragDropEvents: function() {
        this.node.setAttribute("draggable", "true");

        Event.observe(this.node, "dragstart", this.dndHandlers.start);
        Event.observe(this.node, "dragend", this.dndHandlers.end);

        Event.observe(this.node, "dragenter", this.dndHandlers.enter);
        Event.observe(this.node, "dragover", this.dndHandlers.over);
        Event.observe(this.node, "dragleave", this.dndHandlers.leave);
        Event.observe(this.node, "drop", this.dndHandlers.drop);
    },

    _unbindEvents: function() {
        this._unbindDragDropEvents();
        this._unbindMouseEvent();
    },

    _unbindMouseEvent: function() {
        Event.unobserve(this.row, "click", this.mouseHandler);
    },

    _unbindDragDropEvents: function() {
        this.node.removeAttribute("draggable");

        Event.unobserve(this.node, "dragstart", this.dndHandlers.start);
        Event.unobserve(this.node, "dragend", this.dndHandlers.end);

        Event.unobserve(this.node, "dragenter", this.dndHandlers.enter);
        Event.unobserve(this.node, "dragover", this.dndHandlers.over);
        Event.unobserve(this.node, "dragleave", this.dndHandlers.leave);
        Event.unobserve(this.node, "drop", this.dndHandlers.drop);
    },

    _handleMouseEvent: function(e) {
        var target = e.target || e.srcElement || null;

        if (target) {
            if (target == this.markBox) {
                this.listener.onMark(this);
            } else if (target == this.node) {
                Event.stop(e);
                this.listener.onGet(this);
            } else {
                this.listener.onSelect(this, e.ctrlKey ? !this.selected : true);
            }
        }
    },

    _handleKeyboardEvent: function(e) {
        // todo:
    },

    _handleDragStart: function(e) {
        if (FileListView.Item.isChangeDirUp(this)) {
            Event.stop(e);
            return;
        }

        e.dataTransfer.setData("Text", this.path);
        e.dataTransfer.setData("Url", document.location.href);
        e.dataTransfer.effectAllowed = "move";
        e.dataTransfer.dropEffect = "move";

        FileListView.Item.droppedCorrectly = false;
        FileListView.Item.currentlyDragged = this;

        this.listener.onMoveStart(this);
    },

    _handleDragEnd: function() {
        FileListView.Item.currentlyDragged = null;

        if (!FileListView.Item.droppedCorrectly) {
            this.listener.onMoveCancel(this);
        }
    },

    _handleDrop: function(e) {
        Event.stop(e);
        Utils.removeCssClass(this.node, "over");
        FileListView.Item.droppedCorrectly = true;
        FileListView.Item.currentlyDragged = null;

        this.listener.onMove(e.dataTransfer.getData("Text"), this.path);
    },

    _handleDragEnter: function(e) {
        Event.stop(e);
        Utils.addCssClass(this.node, "over");
    },

    _handleDragOver: function(e) {
        if (this._isDroppable()) {
            Event.stop(e);
        }
    },

    _handleDragLeave: function() {
        Utils.removeCssClass(this.node, "over");
    },

    _isDroppable: function() {
        return this.access.move && FileListView.Item.currentlyDragged != this && !this.isMarked();
    }
});
FileListView.Item.fromRow = function(row, listener) {
    var node = row.cells[1].getElementsByTagName("A")[0];
    var inputs = row.getElementsByTagName("INPUT");
    var markBox = inputs.length > 0 && Utils.isCheckboxInput(inputs[0]) ? inputs[0] : null;
    return new FileListView.Item(row, node, markBox, listener)
};
FileListView.Item.isChangeDirUp = function(item) {
    return item.node.id && item.node.id == "cdup";
};

FileListView.Listener = Class.create({
    onChangeDir: function(path) {},
    onDloadFile: function(path) {},
    onMoveFile: function(files, destination) {},
    onSelectionChanged: function(view) {}
});
FileListView.Access = Class.create({

    change: false,
    dload: false,
    move: false,
    view: false,

    _constructor: function(data) {
        this.change = /^d.*$/.test(data);
        this.dload = /^f.*$/.test(data);
        this.move = this.change;
        this.view = /^\w{1}v.*$/.test(data);
    }
});