function TableSorter(table) {
    this._constructor(table);
}(function() { // create local scope;

    TableSorter.TYPE_STRING = 1;
    TableSorter.TYPE_NUMBER = 2;
    TableSorter.TYPE_ERASABLE = 3;

    var ASCENDING = "asc";
    var DESCENDING = "des";
    var K_SORT_ASC = 1;
    var K_SORT_DESC = -1;
    var SORT_ORDER_ATTR = "sortOrder";
    var COLUMN_INDEX_ATTR = "columnIndex";
    var ARROW_UP = "&nbsp;&#x25b2;";
    var ARROW_DOWN = "&nbsp;&#x25bc;";
    var NOT_SORTED = "&nbsp;&nbsp;";
    var CONTROL_CURSOR = "pointer";

    TableSorter.prototype = {

        table: null,
        headerRow: null,
        kSortOrder: null,
        columnTypes: null,
        sortedColumns: null,
        sortedColumnIndex: null,
        erasableHolderClass: null,
        sortFirstClass: null,
        excludedRowIds: null,

        _constructor: function(table) {
            this.table = table;
            this.headerRow = this._getHeaderRow();
            this.kSortOrder = K_SORT_ASC;
            this.columnTypes = new Array();
            this.sortedColumns = new Array();
            this.excludedRowIds = new Array();

            this.setErasableHolderClass(null);
            this.setSortFirstClass(null);
        },

        getSortedColumnIndex: function() {
            return this.sortedColumnIndex;
        },

        isSortedAscending: function() {
            var result = true;

            if (this.headerRow != null && this.sortedColumnIndex != null) {
                var cell = this.headerRow.cells[this.sortedColumnIndex];
                result = (this._getSortOrder(cell) != DESCENDING);
            }

            return result;
        },

        setSortedColumns: function(columns) {
            if (this.getColumnsCount() == 0 || this.getRowsCount() == 0) {
                return;
            }

            for (var i = 0, size = this.sortedColumns.length; i < size; ++i) {
                this.removeSortedColumn(this.sortedColumns[i]);
            }

            var values = [].concat(columns);

            for (var i = 0; i < values.length; ++i) {
                this.addSortedColumn(parseInt(values[i]));
            }
        },

        addSortedColumn: function(columnIndex) {
            if (columnIndex >= 0 && columnIndex < this.getColumnsCount()) {
                var cell = this.headerRow.cells[columnIndex];
                cell.setAttribute(COLUMN_INDEX_ATTR, columnIndex);
                cell.style.cursor = CONTROL_CURSOR;

                var thisObj = this;
                cell.onclick = function() {
                    thisObj.sortByColumn(parseInt(this.getAttribute(COLUMN_INDEX_ATTR)));
                    return false;
                }

                this.sortedColumns[this.sortedColumns.length] = columnIndex;
            }
        },

        removeSortedColumn: function(columnIndex) {
            if (columnIndex >= 0 && columnIndex < this.getColumnsCount()) {
                for (var i = 0, size = this.sortedColumns.length; i < size; ++i) {
                    if (this.sortedColumns[i] == columnIndex) {
                        var cell = this.headerRow.cells[columnIndex];
                        cell.removeAttribute(COLUMN_INDEX_ATTR);
                        cell.style.cursor = null;
                        cell.onclick = null;

                        delete this.sortedColumns[i];
                    }
                }
            }
        },

        getColumnsCount: function() {
            return (this.headerRow != null && this.headerRow.cells) ? this.headerRow.cells.length : 0;
        },

        getRowsCount: function() {
            return (this.table.rows) ? this.table.rows.length : 0;
        },

        getColumnType: function(columnIndex) {
            var result = this.columnTypes[columnIndex]
            return result ? result : TableSorter.TYPE_STRING;
        },

        setColumnType: function(columnIndex, type) {
            this.columnTypes[columnIndex] = type;
        },

        setErasableHolderClass: function(cssClass) {
            this.erasableHolderClass = (cssClass != null) ? cssClass : "";
        },

        setSortFirstClass: function(cssClass) {
            this.sortFirstClass = (cssClass != null) ? cssClass : "";
        },

        excludeRowIds: function(ids) {
            this.excludedRowIds = new Array();
            var values = [].concat(ids);

            for (var i = 0; i < values.length; ++i) {
                this.excludedRowIds[this.excludedRowIds.length] = values[i];
            }
        },

        setSortedBy: function(columnIndex, ascending) {
            // sort only if table contains header and more than 1 row
            if (!(this.getColumnsCount() > 0 && this.getRowsCount() > 2 && this._isSortedColumn(columnIndex))) {
                return;
            }

            this._clearSortOrder();
            this.sortedColumnIndex = columnIndex;
            this.kSortOrder = ascending ? K_SORT_ASC : K_SORT_DESC;

            var control = this.headerRow.cells[this.sortedColumnIndex];
            this._setSortOrder(control, (ascending ? ASCENDING : DESCENDING));
        },

        sortByColumn: function(columnIndex, ascending) {
            // sort only if table contains header and more than 1 row
            if (!(this.getColumnsCount() > 0 && this.getRowsCount() > 2 && this._isSortedColumn(columnIndex))) {
                return;
            }

            var control = this.headerRow.cells[this.sortedColumnIndex];
            control.blur();

            var sortAscending = (arguments.length == 2) ? ascending : !(this._getSortOrder(control) != DESCENDING);
            this.setSortedBy(columnIndex, sortAscending);

            // create sort values array
            var sortValues = this._getSortValues(this.sortedColumnIndex);
            var type = this.getColumnType(this.sortedColumnIndex);
            var thisObj = this;

            switch (type) {
                case TableSorter.TYPE_STRING:
                    sortValues.sort(function(a, b) {
                        return thisObj._sortAsString(a, b);
                    });
                    break;

                case TableSorter.TYPE_NUMBER:
                    sortValues.sort(function(a, b) {
                        return thisObj._sortAsNumber(a, b);
                    });
                    break;

                case TableSorter.TYPE_ERASABLE:
                    sortValues.sort(function(a, b) {
                        return thisObj._sortAsErasable(a, b);
                    });
                    break;
            }

            this._updateTable(sortValues);
            this._clearSortValues(sortValues);
        },

        _updateTable: function(values) {
            var tBody = this.table.tBodies[0];

            if (Utils.Browser.Gecko) {
                // Mozilla is faster when doing
                // the DOM manipulations on an orphaned element.
                var tBodyParent = tBody.parentNode;
                var tBodySibling = tBody.nextSibling;

                tBodyParent.removeChild(tBody);

                for (var i = 0; i < values.length; ++i) {
                    tBody.appendChild(values[i].row);
                }

                tBodyParent.insertBefore(tBody, tBodySibling);
            } else if (Utils.Browser.IEMac) {
                // IE 5 Mac hung up when appending
                // an existing table row to end of its table
                var nextRow = null;
                for (var i = 0; i < tBody.rows.length; ++i) {
                    if (this._isSortedRow(tBody.rows[i])) {
                        nextRow = tBody.rows[i];
                        break;
                    }
                }

                for (var i = 0; i < values.length - 1; ++i) {
                    tBody.insertBefore(values[i].row, nextRow);
                    nextRow = values[i].row.nextSibling;
                }
            } else {
                for (var i = 0; i < values.length; ++i) {
                    tBody.appendChild(values[i].row);
                }
            }
        },

        _sortAsString: function(a, b) {
            var value1 = a.value;
            var value2 = b.value;

            if (a.sortFirst) {
                return b.sortFirst ? Utils.compareIgnoreCase(value1, value2) * this.kSortOrder : -1;
            } else {
                return b.sortFirst ? 1 : Utils.compareIgnoreCase(value1, value2) * this.kSortOrder;
            }
        },

        _sortAsErasable: function(a, b) {
            var value1 = a.value;
            var value2 = b.value;

            if (a.sortFirst) {
                return b.sortFirst ? (value1 - value2) * this.kSortOrder : -1;
            } else {
                return b.sortFirst ? 1 : (value1 - value2) * this.kSortOrder;
            }
        },

        _sortAsNumber: function(a, b) {
            var value1 = parseInt(a.value);
            var value2 = parseInt(b.value);
            value1 = isNaN(value1) ? 0 : value1;
            value2 = isNaN(value2) ? 0 : value2;

            if (a.sortFirst) {
                return b.sortFirst ? this._sortOnFirstStringColumn(a, b) : -1;
            } else {
                return b.sortFirst ? 1 : (value1 - value2) * this.kSortOrder;
            }
        },

        _sortOnFirstStringColumn: function(a, b) {
            // bug #224128 in Firefox
            // Array.sort() not stable on duplicate values
            // sort ascending on the first string column
            var columnIndex = this._columnIndexOfType(TableSorter.TYPE_STRING);

            if (columnIndex != -1) {
                var str1 = this._getValueAt(a.row, columnIndex);
                var str2 = this._getValueAt(b.row, columnIndex);

                return Utils.compareIgnoreCase(str1, str2);
            } else {
                return 0;
            }
        },

        _columnIndexOfType: function(type) {
            for (var i = 0; i < this.columnTypes.length; ++i) {
                var candidate = this.columnTypes[i];

                if (candidate == type) {
                    return i;
                }
            }

            return -1;
        },

        _getSortValues: function(columnIndex) {
            var result = new Array();
            var isErasable = (this.getColumnType(columnIndex) == TableSorter.TYPE_ERASABLE);
            var rows = this.table.rows;

            for (var i = 0; i < rows.length; ++i) {
                var row = rows[i];

                if (this._isSortedRow(row)) {
                    var value;

                    if (isErasable) {
                        var holder = this._getErasableHolder(row.cells[columnIndex]);
                        value = (holder != null) ? parseInt(Utils.getInnerText(holder)) : 0;
                        value = isNaN(value) ? 0 : value;
                    } else {
                        value = this._getValueAt(i, columnIndex);
                    }

                    result[result.length] = {row: row, value: value, sortFirst: this._isSortFirst(row)};
                }
            }

            return result;
        },

        _clearSortValues: function(values) {
            for (var i = 0; i < values.length; ++i) {
                values[i].row = null;
                values[i].value = null;
                values[i] = null;
            }
        },

        _getSortOrderControl: function(elem) {
            var spans = elem.getElementsByTagName(Tag.SPAN);
            return (spans.length == 0) ? elem.appendChild(document.createElement(Tag.SPAN)) : spans[0];
        },

        _getSortOrder: function(elem) {
            var control = this._getSortOrderControl(elem);
            return (control != null) ? control.getAttribute(SORT_ORDER_ATTR) : ASCENDING;
        },

        _setSortOrder: function(elem, order) {
            var control = this._getSortOrderControl(elem);

            if (control != null) {
                control.setAttribute(SORT_ORDER_ATTR, order)
                control.innerHTML = (order == null) ? NOT_SORTED : (order == ASCENDING ? ARROW_UP : ARROW_DOWN);
            }
        },

        _clearSortOrder: function() {
            for (var i = 0, size = this.sortedColumns.length; i < size; ++i) {
                var columnIndex = this.sortedColumns[i];
                this._setSortOrder(this.headerRow.cells[columnIndex], null);
            }
        },

        _getValueAt: function(row, column) {
            var cell = Utils.isRow(row) ? row.cells[column] : this.table.rows[row].cells[column];
            return Utils.getInnerText(cell).trim();
        },

        _getHeaderRow: function() {
            var rows = this.table.rows;

            for (var i = 0; i < rows.length; ++i) {
                var candidate = rows[i];

                if (Utils.isHeaderRow(candidate)) {
                    return candidate;
                }
            }

            return null;
        },

        _isSortedColumn: function(columnIndex) {
            for (var i = 0; i < this.sortedColumns.length; ++i) {
                if (this.sortedColumns[i] == columnIndex) {
                    return true;
                }
            }

            return false;
        },

        _isSortedRow: function(row) {
            if (this.headerRow == row) {
                return false;
            }

            var link = this._getFirstLink(row);

            for (var i = 0; i < this.excludedRowIds.length; ++i) {
                if (this.excludedRowIds[i] == link.id) {
                    return false;
                }
            }

            return true;
        },

        _isSortFirst: function(row) {
            var link = this._getFirstLink(row);
            return Utils.containsCssClass(link, this.sortFirstClass);
        },

        _getErasableHolder: function(elem) {
            for (var el = elem.firstChild; el != null; el = el.nextSibling) {
                if (Utils.containsCssClass(el, this.erasableHolderClass)) {
                    return el;
                }
            }

            return null;
        },

        _getFirstLink: function(elem) {
            var links = elem.getElementsByTagName("a");
            return (links.length > 0) ? links[0] : null;
        }

    }

})();