var Channel = Class.create({

    timeout: 0,
    preventCaching: false,
    timeoutTimer: null,
    interrupted: false,

    listener: null,

    _constructor: function(config) {
        config = Object.extend(Object.extend({}, Channel.DEFAULT_CONFIG), config);
        this.timeout = config.timeout;
        this.preventCaching = config.preventCaching;
    },

    open: function(listener) {
        this.listener = listener || new Channel.Listener();
        this._createTimeoutTimer();
    },

    send: function() {
        throw "Not implemented."
    },

    abort: function() {
        this.interrupted = true;
        this.close();
    },

    close: function() {
        this._stopTimeoutTimer();
        this.timeoutTimer = null;
    },

    _handleTimeout: function() {
        try {
            this.abort();
        } catch (e) {}

        this.listener.onTimeout(this);
    },

    _createTimeoutTimer: function() {
        if (this.timeout > 0) {
            this.timeoutTimer = new Timer(this._handleTimeout.bind(this), this.timeout);
            this.timeoutTimer.setRepeats(false);
        }
    },

    _startTimeoutTimer: function() {
        if (this.timeoutTimer) {
            this.timeoutTimer.start();
        }
    },

    _stopTimeoutTimer: function() {
        if (this.timeoutTimer) {
            this.timeoutTimer.stop();
        }
    },

    _convertToChannelData: function(data, provider) {
        var result = [];
        for (var key in data) {
            if (!data.hasOwnProperty(key)) {
                continue;
            }
            var value = data[key];
            if (Object.isArray(value)) {
                for (var i = 0; i < value.length; ++i) {
                    result.push(provider(key, value[i]));
                }
            } else {
                result.push(provider(key, value));
            }
        }
        return result;
    }

});

Channel.DEFAULT_CONFIG = {timeout: 0, preventCaching: false};
Channel.Listener = Class.create({
    onComplete: function(ch) {},
    onTimeout: function(ch) {},
    onError: function(ch) {}
});

/** HTML iframe transport channel */
var IFrameChannel = Class.create(Channel, {

    url: null,
    method: null,
    name: null,

    form: null,
    iframe: null,
    iframeLoadHandler: null,
    loadCompleteToken: null,
    loadTokenTimer: null,

    responseText: null,
    responseHtml: null,
    responseMetadata: null,

    _constructor: function(config) {
    },

    getUrl: function() {
        return this.url;
    },

    getResponseText: function() {
        return this.responseText;
    },

    getResponseHtml: function() {
        return this.responseHtml;
    },

    getResponseDocument: function() {
        return this._getContentDocument();
    },

    getResponseMetadata: function() {
        return this.responseMetadata || {};
    },

    setLoadCompleteToken: function(value) {
        this.loadCompleteToken = value;
    },

    submitForm: function(form) {
        if (!Object.isElement(form)) {
            throw "Illegal argument form";
        }

        this.form = form;
        this.method = form.method;
        this._sendFormData();
    },

    open: function(url, method, listener) {
        this.url = url || IFrameChannel.BLANK_URL;
        this.method = (method || "get").toUpperCase();
        this._super.open.call(this, listener);
    },

    send: function(data) {
        this._createForm(data);
        this._sendFormData();
    },

    close: function() {
        this._super.close.call(this);

        this.responseText = null;
        this.responseHtml = null;
        this.responseMetadata = null;
        this._dispose();
    },

    _dispose: function() {
        this._disposeForm();
        this._disposeIframe();
    },

    _getContentDocument: function() {
        return this.iframe ? (this.iframe.contentDocument || this.iframe.contentWindow.document) : null;
    },

    _sendFormData: function() {
        this._createIframe();
        this._appendIframe();
        this._bindEvents();

        try {
            this.form.target = this.iframe.name;
            this._scheduleLoadCompleteToken();
            this._startTimeoutTimer();
            this.form.submit();
        } catch(e) {
            this._stopTimeoutTimer();
            this._unbindEvents();
            this._dispose();
        }
    },

    _createForm: function(data) {
        if (this.form == null) {
            var form = document.createElement(Tag.FORM);
            form.setAttribute("action", this._createUrl());
            form.setAttribute("method", this.method);
            form.setAttribute("accept-charset", "utf-8");

            var inputs = this._toFormInputs(data);
            for (var i = 0; i < inputs.length; ++i) {
                form.appendChild(inputs[i]);
            }
            Utils.setStyle(form, {position:"absolute",top:0,left:"-10000px",visibility:"hidden", overflow:"hidden"});
            this.form = document.body.appendChild(form);
        }

        return this.form;
    },

    _disposeForm: function() {
        if (this.form && this.form.parentNode) {
            this.form.parentNode.removeChild(this.form);
        }
        this.form = null;
    },

    _appendIframe: function() {
        document.body.appendChild(this.iframe);
    },

    _createIframe: function() {
        if (this.iframe == null) {
            this.name = "iframe-channel-" + (++IFrameChannel._sequence).toString(36);
            // todo: remove holder
            var holder = document.createElement(Tag.DIV);
            holder.innerHTML = "<iframe " + (Utils.Browser.IE ? "src='javascript:\"\"'" : "")
                    + "id='" + this.name + "' name='" + this.name + "' "
                    + "frameborder='0' " // border bug in IE7
                    + "style='width:0;height:0;border:none;visibility:hidden'></iframe>";
            this.iframe = holder.getElementsByTagName("iframe")[0];
            Utils.setStyle(this.iframe, {position: "relative", top: 0, left: 0});
        }
        return this.iframe;
    },

    _disposeIframe: function() {
        if (!Object.isElement(this.iframe)) {
            return;
        }
        this._unbindEvents();
        var iframe = this.iframe;
        this.iframe = null;

        if (Utils.Browser.Opera || Utils.Browser.Gecko) {
            (function(f) {
                f && f.parentNode && f.parentNode.removeChild(f);
            }).delay(2000, iframe);
        } else {
            if (iframe.parentNode) {
                iframe.parentNode.removeChild(iframe);
            }
        }
    },

    _createUrl: function() {
        var result = encodeURI(this.url);

        if (this.preventCaching) {
            var qs = this._toQueryString({nocache: new Date().getTime()});
            result += (result.indexOf("?") == -1 ? "?" : "&") + qs;
        }

        return result;
    },

    _toQueryString: function(data) {
        return this._convertToChannelData(data, function(key, value) {
            return key + "=" + encodeURIComponent(value);
        }).join("&");
    },

    _toFormInputs: function(data) {
        return this._convertToChannelData(data, function(key, value) {
            var input = Utils.createInput(Tag.INPUT_TYPE_HIDDEN, key);
            input.value = value;
            return input;
        });
    },

    _toCookieValue: function(data) {
        return this._convertToChannelData(data, function(key, value) {
            return key + "=" + value;
        }).join("; ");
    },

    _scheduleLoadCompleteToken: function() {
        if (!Object.isString(this.loadCompleteToken) || !this.form) {
            return;
        }
        var data = {};data[IFrameChannel.LOAD_COMPLETE_TOKEN] = this.loadCompleteToken;
        var token = this._toFormInputs(data).first();
        this.form.appendChild(token);

        this.loadTokenTimer && this.loadTokenTimer.stop();
        this.loadTokenTimer = new Timer(this._handleLoadCompleteToken.bind(this), 1000);
        this.loadTokenTimer.start();
    },

    _handleLoadCompleteToken: function() {
        var cookieName = IFrameChannel.LOAD_COMPLETE_TOKEN;
        var token = Utils.getCookieValue(cookieName);

        if (token && token == this.loadCompleteToken) {
            Utils.setCookieValue(cookieName, "", 0);
            this.loadCompleteToken = null;
            this._handleLoadComplete();
        }
    },

    _handleLoadComplete: function() {
        this._stopTimeoutTimer();

        try {
            this._parseResponse(this._getContentDocument());
            this.listener.onComplete(this);
        } catch(e) {
            this.listener.onError(this);
        }

        this._dispose();
    },

    _handleLoadCompleteOpera: function() {
        if (Utils.Browser.Opera && this._getContentDocument().location == IFrameChannel.BLANK_URL) {
            return;
        }
        this._handleLoadComplete();
    },

    _parseResponse: function(doc) {
        if (doc && doc.body) {
            this.responseText = doc.body.textContent || doc.body.innerText;
            this.responseHtml = doc.body.innerHTML;
            this.responseMetadata = {};

            var tags = doc.getElementsByTagName("meta");
            for (var i = 0; i < tags.length; ++i) {
                var metadata = tags[i];
                this.responseMetadata[metadata.httpEquiv] = metadata.content;
            }
        } else {
            this.responseText = "";
            this.responseHtml = "";
            this.responseMetadata = {};
        }
    },

    _bindEvents: function() {
        if (this.iframe) {
            this.iframeLoadHandler = Utils.Browser.Opera ? this._handleLoadCompleteOpera.bind(this) : this._handleLoadComplete.bind(this);
            Event.observe(this.iframe, "load", this.iframeLoadHandler);
        }
    },

    _unbindEvents: function() {
        if (this.iframe) {
            Event.unobserve(this.iframe, "load", this.iframeLoadHandler);
            this.iframeLoadHandler = null;
        }
    },

    _stopTimeoutTimer: function() {
        this._super._stopTimeoutTimer.call(this);

        if (this.loadTokenTimer) {
            this.loadTokenTimer.stop();
            this.loadTokenTimer = null;
        }
    }
});
IFrameChannel._sequence = 0;
IFrameChannel.BLANK_URL = "about:blank";
IFrameChannel.LOAD_COMPLETE_TOKEN = "x-load-complete";

/** HTML script transport channel */
var ScriptChannel = Class.create(Channel, {

    url: null,
    scripts: null,
    timers: null,

    _constructor: function(config) {
        this.scripts = [];
        this.timers = [];
    },

    open: function(url, listener) {
        this.url = url;
        this.listener = listener || new Channel.Listener();
    },

    send: function(data, callback) {
        data = data || {};
        var properties = {};
        this._bindCallbackHandler(callback, properties);
        var script = this._createScript(properties);
        script.src = this._createUrl(properties.callback ? Object.extend(data, {callback: properties.callback}) : data);

        this._bindEvents(script, properties);
        this._startTimeoutTimer(script, properties);
        this._enqueueScript(script);
    },

    close: function() {
        while (this.timers.length != 0) {
            this._stopTimeoutTimer(this.timers.shift());
        }
        while (this.scripts.length != 0) {
            this._cleanup(this.scripts.shift());
        }
    },

    _cleanup: function(script) {
        this._unbindEvents(script);

        var ref = window[ScriptChannel._name];
        script.id && ref[script.id] && delete ref[script.id];

        if (script && script.parentNode) {
            script.parentNode.removeChild(script);
        }
    },

    _bindCallbackHandler: function(callback, properties) {
        if (Object.isFunction(callback)) {
            var ref = ScriptChannel._name;
            var id = ScriptChannel._generateIdentifier();
            window[ref][id] = function(f) {
                this[id] && delete this[id];
                f.apply(f, Array.prototype.slice.call(arguments, 1));
            }.bind(window[ref], callback);

            properties.id = id;
            properties.callback = ref + "." + id;
        }
    },

    _handleLoaded: function(script, props) {
        if (this._scriptLoaded(script)) {
            this._stopTimeoutTimer(props.timer);
            this._cleanup(script);
            this._dequeueScript(script);
        }
    },

    _handleLoadError: function(script, props) {
        this._stopTimeoutTimer(props.timer);
        this._cleanup(script);
        this._dequeueScript(script);
        this.listener.onError(this);
    },

    _handleTimeout: function(script) {
        this._cleanup(script);
        try {
            this.abort();
        } catch (e) {}
        this.listener.onTimeout(this);
    },

    _startTimeoutTimer: function(script, props) {
        if (this.timeout > 0) {
            props.timer = (function(self, sc) {
                self._handleTimeout(sc);
            }).delay(this.timeout, this, script);
            this.timers.push(props.timer);
        }
    },

    _stopTimeoutTimer: function(timer) {
        timer && window.clearTimeout(timer);
    },

    _scriptLoaded: function (script) {
        return !script.readyState || /loaded|complete/.test(script.readyState);
    },

    _bindEvents: function(script, props) {
        var globalErrorHandler = window.onerror;
        var callbackFunctionName = props.callback;

        script.onload = script.onreadystatechange = function() {
            window.onerror = globalErrorHandler;
            this._handleLoaded(script, props);
        }.bind(this);
        script.onerror = function() {
            window.onerror = globalErrorHandler;
            this._handleLoadError(script, props);
        }.bind(this);
        window.onerror = function(message, url) {
            window.onerror = globalErrorHandler;
            if (script.src === url || url.indexOf(callbackFunctionName) != -1) {
                this._handleLoadError(script, props);
                return true;
            }
            return false;
        }.bind(this);
    },

    _unbindEvents: function(script) {
        if (Object.isElement(script)) {
            script.onload = null;
            script.onerror = null;
            script.onreadystatechange = null;
        }
    },

    _enqueueScript: function(script) {
        var holder = this._scriptHolder();
        this.scripts.push(holder.insertBefore(script, holder.firstChild));
    },

    _dequeueScript: function(script) {
        var index = this.scripts.indexOf(script);
        if (index != -1) {
            this.scripts[index] = null;
            this.scripts.splice(index, 1);
        }
    },

    _scriptHolder: function() {
        return document.head || document.getElementsByTagName("HEAD")[0] || document.documentElement;
    },

    _createScript: function(props) {
        var el = document.createElement("script");
        el.type = "text/javascript";
        el.charset = "UTF-8";
        el.async = "async";
        props && props.id && (el.id = props.id);
        return el;
    },

    _createUrl: function(data) {
        if (this.preventCaching) {
            data["nocache"] = new Date().getTime();
        }
        var uri = encodeURI(this.url);
        var qs = this._toQueryString(data);
        return qs.length > 0 ? uri + (uri.indexOf("?") == -1 ? "?" : "&") + qs : uri;
    },

    _toQueryString: function(data) {
        return this._convertToChannelData(data, function(key, value) {
            return key + "=" + encodeURIComponent(value);
        }).join("&");
    }
});
(function() {
    this._name = "__sdc";
    this._sequence = 0;
    this._generateIdentifier = function(){ return "_" + (this._sequence++).toString(36) + new Date().getTime().toString(36)};
    window[this._name] = {};
}).call(ScriptChannel);