var Uploader = Class.create({

    uploadForm: null,
    uploadView: null,
    progressView: null,

    uploadQueue: null,
    commandChannel: null,
    progressMeter: null,
    progressListener: null,

    metadata: null,
    sessionId: null,
    interrupted: false,
    transferError: null,
    listener: null,

    _constructor: function(config) {
        this.metadata = config.metadata || {};

        this.uploadForm = new jscape.UploadForm(config.ufu, this.metadata, config.uploadFormRequired);
        this.uploadForm.setForms(config.uploadForms || []);
        this.uploadView =  new jscape.UploadView(new FormFileChooser());

        this.progressMeter = new jscape.ProgressMeter(config);
        this.progressView = new jscape.ProgressView({
            onCancelUpload: this._handleUploadCancel.bind(this)
        });
        this.progressListener = {
            onProgress: this._handleProgressProgress.bind(this),
            onComplete: this._handleProgressComplete.bind(this),
            onCancel: this._handleProgressCancel.bind(this),
            onTimeout: this._handleTimeout.bind(this),
            onError: this._handleProgressError.bind(this)
        };
        this.commandChannel = new ScriptChannel({timeout: config.connectionTimeout, preventCaching: true});
        this.commandChannel.open(config.usu, Object.extend(new Channel.Listener(), {
            onError: this._handleError.bind(this)
        }));
    },

    getTransferError: function() {
        return this.transferError;
    },

    isInterrupted: function() {
        return this.interrupted;
    },

    start: function(listener) {
        this.listener = listener || new Uploader.Listener();
        this.uploadView.show({
            onReady: this._handleReadyState.bind(this),
            onChooseFiles: this._handleChooseFiles.bind(this),
            onRemoveAll: this._handleRemoveAll.bind(this),
            onStartUpload: this._handleStartUpload.bind(this)
        });
    },

    upload: function() {
        this.interrupted = false;
        this.uploadView.hide();
        this.progressView.setVisible(true);
        this.listener.onStart(this);

        this._initialHandshake();
    },

    cancel: function() {
        this.uploadQueue.abort();
        this.progressMeter.interrupt();
        this.progressView.setEnabled(false);
    },

    _initialHandshake: function() {
        try {
            this._sendCommand({
                createSession: true,
                client: this.uploadQueue.client
            }, function(response) {
                this._createSession(response.id, response.url);
            }.bind(this));
        } catch (e) {
            this._handleError();
        }
    },

    _createSession: function(id, uploadUrl) {
        this.sessionId = id;
        this._uploadOperation(uploadUrl);
        this.progressMeter.start(this.progressListener, this.sessionId);
    },

    _uploadOperation: function(url) {
        this.uploadQueue.send(url, {
            onComplete: this._handleUploadComplete.bind(this),
            onError: this._handleError.bind(this)
        });
    },

    _cancelOperation: function() {
        this._sendCommand({cancelSession: this.sessionId});
    },

    _sendCommand: function(command, callback) {
        try {
            this.commandChannel.send(Object.extend(command, this.metadata), callback);
        } catch (e) {}
    },

    _handleUploadComplete: function() {
        this.transferError = null;
        this.progressView.setProgress(100);
        this.listener.onComplete(this);
    },

    _handleUploadCancel: function() {
        this.interrupted = true;
        this.cancel();
        this._cancelOperation();
        this.progressView.setVisible(false);

        this.listener.onComplete(this);
    },

    _handleTimeout: function() {
        this.cancel();
        this.uploadView._showConnectionTimeoutMessage();
        this.listener.onTimeout(this);
    },

    _handleError: function() {
        this.cancel();
        this.transferError = Uploader.Messages.TRANSFER_ERROR;
        this.progressView.setVisible(false);
        this.listener.onComplete(this);
    },

    _handleProgressComplete: function() {
        this.progressMeter.finish();
        this.progressView.setEnabled(false);
    },

    _handleProgressError: function(message) {
        this.cancel();
        this.transferError = message;
        this.uploadView._showConnectionErrorMessage(message);
        this.listener.onComplete(this);
    },

    _handleProgressProgress: function(percent, statusMessage, timeMessage) {
        this.progressView.setProgress(percent);
        this.progressView.setStatus(statusMessage);
        this.progressView.setTime(timeMessage);
    },

    _handleProgressCancel: function() {
        this.progressMeter.interrupt();
        this.progressView.setEnabled(false);
    },

    _handleReadyState: function(chooser) {
        this.uploadQueue = chooser.createUploadQueue();
    },

    _handleChooseFiles: function(files) {
        for (var i = 0; i < files.length; ++i) {
            if (this.uploadQueue.add(files[i])) {
                this.uploadView.addFile(files[i]);
            }
        }
    },

    _handleRemoveAll: function() {
        this.uploadQueue.clear();
        this.uploadView.clear();
    },

    _handleStartUpload: function() {
        if (this.uploadQueue.isEmpty()) {
            this.uploadView._showNoFilesSelectedMessage();
            return;
        }

        var duplicates = this._retainQueuedWithRemote();

        if (duplicates.length != 0) {
            this.uploadView._showConfirmOverwriteMessage(duplicates, this._onConfirmOverwrite.bind(this));
        } else {
            this._startUpload();
        }
    },

    _startUpload: function() {
        if (!this.uploadQueue.isEmpty()) {
            var filenames = [];
            var fileItems = this.uploadQueue.fileItems();
            for (var i = 0; i < fileItems.length; ++i) {
                filenames.push(fileItems[i].filename);
            }

            this.uploadForm.start(filenames, {
                onStartUpload: this.upload.bind(this),
                onCancelUpload: this.cancel.bind(this)
            });
        }
    },

    _onConfirmOverwrite: function(confirmed, files) {
        if (!confirmed) {
            while (files.length != 0) {
                var item = files.pop();
                if (this.uploadQueue.remove(item)) {
                    this.uploadView.removeFile(item);
                }
            }
        }
        this._startUpload();
    },

    _retainQueuedWithRemote: function() {
        var fileItems = this.uploadQueue.fileItems();
        var remoteFiles = this._listRemoteFilenames();
        var duplicateItems = [];

        for (var i = 0; i < fileItems.length; ++i) {
            if (remoteFiles.contains(fileItems[i].filename)) {
                duplicateItems.push(fileItems[i]);
            }
        }

        return duplicateItems;
    },

    // todo: refactor
    _listRemoteFilenames: function() { return window.listFiles(); }
});
Uploader.Listener = Class.create({
    onStart: function() {},
    onComplete: function() {},
    onTimeout: function() {}
});
Uploader.Messages = {
    NO_FILES_SELECTED: "No files selected.",
    TRANSFER_ERROR: "Transfer error.",
    CONNECTION_TIMEOUT: "Connection timeout.",
    CONFIRM_OVERWRITE: "The target file(s) already exists. Overwrite?"
};
