jscape.domain = jscape.domain || {};

/** @class */
jscape.IpAccessRule = Class.extend(
    /** @lends {jscape.IpAccessRule.prototype} */
{
    /** @constructor */
    initialize: function(a, b, c) {
        this.mask = a || "";
        this.reason = b || "";
        this.allowed = c || false;
    }
});
jscape.IpAccessRule.fromJSON = function(data) {
    return new jscape.IpAccessRule(data.mask, data.reason, data.allowed);
};

/** @class */
jscape.domain.IPAccessModule = jscape.domain.DomainModule.extend(
    /** @lends {jscape.domain.IPAccessModule.prototype} */
{
    onCreate: function () {
        this._super.apply(this, arguments);

        this.controller = new jscape.domain.IpAccessController(this.resources);
    },

    onStart: function () {
        this.controller.start(this.managementService, this.domainName);
    },

    onPause: function() {
        if (this.controller.hasChanged()) {
            this._confirmApply()
                    .done($.proxy(function () { this.controller.onApply(); }, this));
        }
    },

    onResume: function() {
        this.controller._loadData();
    }
});

/** @class */
jscape.domain.IpAccessController = Class.extend(
    /** @lends {jscape.domain.IpAccessController.prototype} */
{
    rules: null,
    copy: null,

    /** @constructor */
    initialize: function(resources) {
        this.rules = [];
        this.resources = resources;

        this.view = new jscape.domain.IpAccessView(resources);
        this.addController = new jscape.domain.AddIpAccessController(resources);
        this.editController = new jscape.domain.EditIpAccessController(resources);
        this.deleteController = new jscape.domain.DeleteIpAccessController(resources);

        this._initValidators(resources);
    },

    start: function(managementService, domainName) {
        this.managementService = managementService;
        this.domainName = domainName;
        this.view.show(this);
    },

    hasChanged: function() {
        return this.copy && !Object._equals(this.copy, this.rules);
    },

    onAddRule: function() {
        this.addController.start()
                .done($.proxy(this._ruleAdded, this));
    },

    onEditRule: function(rule) {
        var index = $.inArray(rule, this.rules);
        if (index != -1) {
            this.editController.start(rule)
                    .done($.proxy(this._ruleEdited, this, index));
        }
    },

    onDeleteRule: function(rules) {
        this.deleteController.start(rules)
                .done($.proxy(this._ruleDeleted, this, rules));
    },

    onRuleUp: function (rule) {
        var index = $.inArray(rule, this.rules);
        if (index > 0) {
            this.rules.splice(index - 1, 2, this.rules[index], this.rules[index - 1]);
            this.view.setRules(this.rules);
            this.view.selectRule(rule);
        }
    },

    onRuleDown: function (rule) {
        var index = $.inArray(rule, this.rules);
        if (index != -1 && index < this.rules.length - 1) {
            this.rules.splice(index, 2, this.rules[index + 1], this.rules[index]);
            this.view.setRules(this.rules);
            this.view.selectRule(rule);
        }
    },

    onRefresh: function() {
        this._loadData();
    },

    onApply: function() {
        if (!this.view.valid()) {
            return $.Deferred().reject().promise();
        }

        var rules = this.rules;
        return this.managementService.setIpAccessRules(this.domainName, rules)
                .done($.proxy(function() {
                    this.copy = this._copyRules(rules);
                }, this))
                .done($.proxy(this._fireApplySuccess, this));
    },

    onDiscard: function() {
        this.rules = this.copy;
        this.copy = this._copyRules(this.rules);
        this.view.setRules(this.rules);
    },

    _ruleAdded: function (rule) {
        this.rules.push(rule);
        this.view.setRules(this.rules);
        this.view.selectRule(rule);
    },

    _ruleEdited: function (index, rule) {
        this.rules[index] = rule;
        this.view.setRules(this.rules);
        this.view.selectRule(rule);
    },

    _ruleDeleted: function (rules) {
        var index = -1;
        while (rules.length) {
            index = $.inArray(rules.pop(), this.rules);
            this.rules.splice(index, 1);
        }
        this.view.setRules(this.rules);

        if (this.rules.length) {
            index = Math.max(0, Math.min(index, this.rules.length - 1));
            this.view.selectRule(this.rules[index]);
        }
    },

    _loadData: function() {
        this.managementService.getIpAccessRules(this.domainName)
                .done($.proxy(function(rules) {
                    this.rules = rules;
                    this.copy = this._copyRules(rules);
                    this.view.setRules(rules);
                }, this));
    },

    _copyRules: function(rules) {
        return $.map(rules, function(rule) { return $.extend(new jscape.IpAccessRule(), rule); });
    },

    _fireApplySuccess: function () {
        $(document).triggerHandler("broadcast", [this.resources.getString("APPLICATION_SUCCESS_APPLY")]);
    },

    _initValidators: function(resources) {
        var v = new jscape.Validator();
        v.nonEmptyRule("ipaccess_mask_nonempty", resources.getString("IP_ACCESS_ERROR_BAD_MASK"));
        v.rule("ipaccess_unique", $.proxy(this._ruleNotExists, this), resources.getString("IP_ACCESS_ERROR_ALREADY_EXIST"));
    },

    _ruleNotExists: function(value, excludes) {
        return $.grep(this.rules, function(rule) {
            return rule.mask == value && (excludes == undefined || $.inArray(rule.mask, excludes) == -1);
        }).length == 0;
    }
});

/** @class */
jscape.domain.IpAccessView = Class.extend(
    /** @lends {jscape.domain.IpAccessView.prototype} */
{
    selection: null,

    /** @constructor */
    initialize: function(resources, container) {
        this.selection = [];
        this.resources = resources;

        this.fieldset = $("fieldset[name=ipaccessfields]", container);

        this.data = $("table[role=grid][aria-label=ipaccess]", this.fieldset);
        this.data.datagrid({
            singleSelect: true,
            fitColumns: true,
            pagination: true,
            search: true,
            columns: [[
                {field: "mask", title: $("thead th:nth-child(1)", this.data).text(), width: 60},
                {field: "allowed", title: $("thead th:nth-child(2)", this.data).text(), width: 15, formatter: $.proxy(this._formatAccess, this), styler: $.proxy(this._styleAccess, this)},
                {field: "reason", title: $("thead th:nth-child(3)", this.data).text(), width: 25}
            ]],
            onSelect: $.proxy(this._adjustRowSelection, this),
            onUnselect: $.proxy(this._adjustRowSelection, this),
            onLoadSuccess: $.proxy(this._adjustRowSelection, this),
            onDblClick: $.proxy(this._onClick$EditButton, this)
        });

        this.moveUpButton = $("a[role=button][name=moveup]", this.fieldset);
        this.moveUpButton.linkbutton({disabled: true, onClick: $.proxy(this._onClick$MoveUpButton, this)});

        this.moveDownButton = $("a[role=button][name=movedown]", this.fieldset);
        this.moveDownButton.linkbutton({disabled: true, onClick: $.proxy(this._onClick$MoveDownButton, this)});

        this.addButton = $("a[role=button][name=add]", this.fieldset);
        this.addButton.linkbutton({onClick: $.proxy(this._onClick$AddButton, this)});

        this.editButton = $("a[role=button][name=edit]", this.fieldset);
        this.editButton.linkbutton({disabled: true, onClick: $.proxy(this._onClick$EditButton, this)});

        this.deleteButton = $("a[role=button][name=delete]", this.fieldset);
        this.deleteButton.linkbutton({disabled: true, onClick: $.proxy(this._onClick$DeleteButton, this)});

        this.refreshButton = $("a[role=button][name=refresh]", this.fieldset);
        this.refreshButton.linkbutton({onClick: $.proxy(this._onClick$RefreshButton, this)});

        this.applyButton = $("a[role=button][name=apply]", this.fieldset);
        this.applyButton.linkbutton({onClick: $.proxy(this._onClick$ApplyButton, this)});

        this.discardButton = $("a[role=button][name=discard]", this.fieldset);
        this.discardButton.linkbutton({onClick: $.proxy(this._onClick$DiscardButton, this)});
    },

    setRules: function(values) {
        this.selection = [];
        this.data.datagrid("loading");
        this.data.datagrid("loadData", values);
        this.data.datagrid("loaded");
    },

    selectRule: function(rule) {
        this.data.datagrid("selectRow", this.data.datagrid("getRowIndex", rule));
    },

    show: function(listener) {
        this.listener = listener || jscape.domain.IpAccessView.LISTENER;
        this._adjustButtonsState();
    },

    valid: function() {
        return this.fieldset.form("validate");
    },

    _formatAccess: function(value) {
        return this.resources.getString(!!value ? "IP_ACCESS_RULE_ALLOWED" : "IP_ACCESS_RULE_DENIED");
    },

    _styleAccess: function(value) {
        return !!value ? "color:green" : "color:red";
    },

    _onClick$AddButton: function() {
        this.listener.onAddRule();
    },

    _onClick$EditButton: function() {
        this.listener.onEditRule(this.selection[0]);
    },

    _onClick$DeleteButton: function() {
        this.listener.onDeleteRule(this.selection);
    },

    _onClick$MoveUpButton: function() {
        this.listener.onRuleUp(this.selection[0]);
    },

    _onClick$MoveDownButton: function() {
        this.listener.onRuleDown(this.selection[0]);
    },

    _onClick$RefreshButton: function() {
        this.listener.onRefresh();
    },

    _onClick$ApplyButton: function() {
        this.applyButton.linkbutton("disable");
        this.listener.onApply(this)
                .always($.proxy(function() { this.applyButton.linkbutton("enable"); }, this));
    },

    _onClick$DiscardButton: function() {
        this.listener.onDiscard();
    },

    _adjustRowSelection: function() {
        this.selection = this.data.datagrid("getSelections");
        this._adjustButtonsState();
    },

    _adjustButtonsState: function() {
        var index = this.selection.length == 1 ? this.data.datagrid("getRowIndex", this.selection[0]) : -1;
        var count = this.data.datagrid("getRows").length;

        this.moveUpButton.linkbutton(index > 0 && index <= count - 1 ? "enable" : "disable");
        this.moveDownButton.linkbutton(index >= 0 && index < count - 1 ? "enable" : "disable");
        this.editButton.linkbutton(this.selection.length == 1 ? "enable" : "disable");
        this.deleteButton.linkbutton(this.selection.length !=0 ? "enable" : "disable");
    }
});

jscape.domain.IpAccessView.LISTENER = {
    onAddRule: $.noop,
    onEditRule: $.noop,
    onDeleteRule: $.noop,
    onRuleUp: $.noop,
    onRuleDown: $.noop,
    onRefresh: $.noop,
    onApply: $.noop,
    onDiscard: $.noop
};

/** @class */
jscape.domain.IpAccessRuleDialog = Class.extend(
    /** @lends {jscape.domain.IpAccessRuleDialog.prototype} */
{
    /** @constructor */
    initialize: function(resources) {
        this.dialog = $("#d-ipaccess-rule").clone().attr("id", new Date().getTime().toString(36));
        this.dialog.dialog({
            width: 530, height: "auto",
            modal: true, resizable: false, closable: true, closed: true,
            buttons: [
                {text: resources.getString("DIALOG_BUTTON_OK") || "Ok", handler: $.proxy(this._onSubmit, this)},
                {text: resources.getString("DIALOG_BUTTON_CANCEL") || "Cancel", handler: $.proxy(this._onCancel, this)}
            ],
            onClose: $.proxy(function () {if (!this.closing) { this._onCancel(); }}, this)
        }).show();

        this.fieldset = $("fieldset[name=ipaccessrulefields]", this.dialog);

        this.mask = $("input[name=mask]", this.fieldset);
        this.mask.validatebox({required: true, validType: ["ipaccess_mask_nonempty", "ipaccess_unique"], missingMessage: resources.getString("IP_ACCESS_ERROR_BAD_MASK")});

        this.reason = $("input[name=reason]", this.fieldset);
        this.allowed = $("input[name=allowed]:radio", this.fieldset);
    },

    setTitle: function(title) {
        this.dialog.dialog("setTitle", title || "");
    },

    show: function (listener) {
        this.listener = listener;
        this.dialog.dialog("center");
        this.dialog.dialog("open");

        this.fieldset.form("validate");
        this.mask.focus();
    },

    hide: function () {
        if (!this._closed()) {
            this.closing = true;
            this.dialog.dialog("close");
            this.fieldset.form("disableValidation");
            this.closing = false;
        }
    },

    destroy: function() {
        this.dialog.dialog("destroy");
    },

    getMask: function () {
        return this.mask.val();
    },

    setMask: function (value) {
        this.mask.val(value);
        if (value && value.trim().length) {
            this.mask.validatebox("options").validType = ["ipaccess_mask_nonempty", "ipaccess_unique['" + value + "']"];
        }
    },

    getReason: function () {
        return this.reason.val();
    },

    setReason: function (value) {
        this.reason.val(value);
    },

    getAllowed: function () {
        return (/^true$/i).test(this.allowed.filter(":checked").val());
    },

    setAllowed: function (value) {
        this.allowed.filter("[value=" + Boolean(value) + "]").prop("checked", true);
    },

    _onSubmit: function () {
        if (this.fieldset.form("validate")) {
            this.hide();
            this.listener.onSubmit(this);
        }
    },

    _onCancel: function () {
        this.hide();
        this.listener.onCancel(this);
    },

    _closed: function () {
        return !!this.dialog.dialog("options").closed;
    }
});

/** @class */
jscape.domain.AddIpAccessController = Class.extend(
    /** @lends {jscape.domain.AddIpAccessController.prototype} */
{
    /** @constructor */
    initialize: function(resources) {
        this.resources = resources;
    },

    start: function() {
        var deferred = $.Deferred();

        var dialog = new jscape.domain.IpAccessRuleDialog(this.resources);
        dialog.setTitle(this.resources.getString("IP_ACCESS_DIALOG_ADD_TITLE"));
        dialog.setAllowed(true);
        dialog.show({
            onSubmit: function(d) {
                var rule = new jscape.IpAccessRule(d.getMask(), d.getReason(), d.getAllowed());
                d.destroy();
                deferred.resolve(rule);
            },
            onCancel: function(d) {
                d.destroy();
                deferred.reject();
            }
        });

        return deferred.promise();
    }
});

/** @class */
jscape.domain.EditIpAccessController = Class.extend(
    /** @lends {jscape.domain.EditIpAccessController.prototype} */
{
    /** @constructor */
    initialize: function(resources) {
        this.resources = resources;
    },

    start: function(rule) {
        var deferred = $.Deferred();

        var dialog = new jscape.domain.IpAccessRuleDialog(this.resources);
        dialog.setTitle(this.resources.getString("IP_ACCESS_DIALOG_EDIT_TITLE"));
        dialog.setMask(rule.mask);
        dialog.setReason(rule.reason);
        dialog.setAllowed(rule.allowed);
        dialog.show({
            onSubmit: function(d) {
                var rule = new jscape.IpAccessRule(d.getMask(), d.getReason(), d.getAllowed());
                d.destroy();
                deferred.resolve(rule);
            },
            onCancel: function(d) {
                d.destroy();
                deferred.reject();
            }
        });

        return deferred.promise();
    }
});

/** @class */
jscape.domain.DeleteIpAccessController = Class.extend(
    /** @lends {jscape.domain.DeleteIpAccessController.prototype} */
{
    /** @constructor */
    initialize: function(resources) {
        this.resources = resources;
    },

    start: function(rules) {
        var text = $.map(rules, function(rule) { return rule.mask; }).join(", ");
        return jscape.ConfirmDialog.confirm(
                this.resources.getString("IP_ACCESS_CONFIRM_DELETE_TITLE"),
                this.resources.getString("IP_ACCESS_CONFIRM_DELETE_MESSAGE", {0: text}));
    }
});