jscape.domain = jscape.domain || {};

/** @class */
jscape.domain.LogBuffer = Class.extend(
    /** @lends {jscape.LogBuffer.prototype} */
{
    DEFAULT_MAX_RECORDS: 1000,
    EOL: "\r\n",

    /** @constructor */
    initialize: function(a) {
        this.maxRecords = parseInt(a) || this.DEFAULT_MAX_RECORDS;
        this.data = "";
    },

    append: function(text) {
        this.data += text;
        this._removeSuperfluousRecords();
    },

    getData: function(recordsCount) {
        var index = this._indexOf(recordsCount);
        return index > -1 ? this.data.substring(index + this.EOL.length) : this.data;
    },

    clear: function () {
        this.data = "";
    },

    _removeSuperfluousRecords: function() {
        var index = this._indexOf(this.maxRecords);
        if (index > -1) {
            this.data = this.data.substring(index + this.EOL.length);
        }
    },

    _indexOf: function(recordsCount) {
        var index = this.data.length - 1;
        while (--recordsCount >= 0 && index > -1) {
            index = this.data.lastIndexOf(this.EOL, index - this.EOL.length);
        }
        return index;
    }
});

/** @class */
jscape.domain.LoggingModule = jscape.domain.DomainModule.extend(
    /** @lends {jscape.domain.LoggingModule.prototype} */
{
    /** @const */
    DELAY: 5 * 1000,

    timerId: null,

    onCreate: function () {
        this._super.apply(this, arguments);

        this.eventLogController = new jscape.domain.EventLogController(this.resources);
        this.searchController = new jscape.domain.LogSearchController(this.resources);
        this.serviceController = new jscape.domain.LogServiceController(this.resources);
        this.syslogController = new jscape.domain.SyslogController(this.resources);
        this.configController = new jscape.domain.LogConfigController(this.resources);
    },

    onStart: function () {
        this.eventLogController.start(this.managementService, this.domainName);
        this.searchController.start(this.managementService, this.domainName);
        this.serviceController.start(this.managementService, this.domainName);
        this.syslogController.start(this.managementService, this.domainName);
        this.configController.start(this.managementService, this.domainName);
    },

    onPause: function () {
        this._stopPoll();

        var serviceChanged = this.serviceController.hasChanged();
        var syslogChanged = this.syslogController.hasChanged();
        var configChanged = this.configController.hasChanged();

        if (serviceChanged || syslogChanged || configChanged) {
            this._confirmApply()
                    .done($.proxy(function () {
                        serviceChanged && this.serviceController.onApply();
                        syslogChanged && this.syslogController.onApply();
                        configChanged && this.configController.onApply();
                    }, this));
        }
    },

    onResume: function () {
        this._poll();

        this.eventLogController._loadData();
        this.searchController._loadData();
        this.serviceController._loadData();
        this.syslogController._loadData();
        this.configController._loadData();
    },

    _poll: function (delay) {
        this._stopPoll();
        this.timerId = setTimeout($.proxy(function () {
            this.timerId = null;
            this._loadDomainStatistics()
                    .done($.proxy(this._poll, this, this.DELAY))
                    .fail($.proxy(this._stopPoll, this));
        }, this), delay || 0);
    },

    _stopPoll: function () {
        if (this.timerId) {
            clearInterval(this.timerId);
        }
    },

    _loadDomainStatistics: function () {
        return this.managementService.getDomainStatistics(this.domainName)
                .done($.proxy(function (statistics) {
                    this.eventLogController.update(statistics);
                }, this));
    }
});

/** @class */
jscape.domain.EventLogController = Class.extend(
    /** @lends {jscape.domain.EventLogController.prototype} */
{
    logBuffer: null,

    /** @constructor */
    initialize: function (resources, container) {
        this.view = new jscape.domain.EventLogView(resources, container);
        this.logBuffer = new jscape.domain.LogBuffer(this.view.getMaxRecords());
    },

    start: function (managementService, domainName) {
        this.managementService = managementService;
        this.domainName = domainName;
        this.view.show(this);
    },

    update: function(stats) {
        this.logBuffer.append(stats.log || "");
        this._setupView();
    },

    onMaxRecordsChanged: function() {
        this._setupView();
    },

    onResumeLog: function() {
        this._setupView();
    },

    _loadData: function() {
        this.managementService.getLogs(this.domainName, this.view.getMaxRecords())
                .done($.proxy(function(data) {
                    this.logBuffer.clear();
                    this.logBuffer.append(data || "");
                    this._setupView();
                }, this));
    },

    _setupView: function() {
        var data = this.logBuffer.getData(this.view.getMaxRecords());
        this.view.setLog(data);
    }
});

/** @class */
jscape.domain.EventLogView = Class.extend(
    /** @lends {jscape.domain.EventLogView.prototype} */
{
    /** @constructor */
    initialize: function(resources, container) {
        this.fieldset = $("fieldset[name=eventlogfields]", container);

        this.maxRecords = $("input[name=maxlogrecords]", this.fieldset);
        this.maxRecords.numberspinner({required: true, min: 10, max: 1000, increment: 10, value: 1000, width: 100, onChange: $.proxy(this._onChange$MaxRecords, this)});

        this.log = $("textarea[name=eventlogs]", this.fieldset);
        this.log.css({width: "77em", height: "22em"}).prop("readonly", true);

        this.pauseButton = $("a[role=button][name=pauselog]", this.fieldset);
        this.pauseButton.linkbutton({toggle: true, onClick: $.proxy(this._onClick$PauseButton, this)});
    },

    getMaxRecords: function() {
        return parseInt(this.maxRecords.numberspinner("getValue"));
    },

    setLog: function(value) {
        if (!this.isPaused()) {
            this.log.val(value);
        }
    },

    isPaused: function() {
        return this.pauseButton.linkbutton("options").selected;
    },

    show: function(listener) {
        this.listener = listener || jscape.domain.EventLogView.LISTENER;
    },

    _onChange$MaxRecords: function() {
        this.listener.onMaxRecordsChanged(this);
    },

    _onClick$PauseButton: function() {
        var text = this.pauseButton.text();
        this.pauseButton.html(this.pauseButton.html().replace(text, this.pauseButton.data("toggle-text")));
        this.pauseButton.data("toggle-text", text);

        if (!this.isPaused()) {
            this.listener.onResumeLog(this);
        }
    }
});
jscape.domain.EventLogView.LISTENER = {
    onMaxRecordsChanged: $.noop,
    onResumeLog: $.noop
};

/** @class */
jscape.LogService = Class.extend(
    /** @lends {jscape.LogService.prototype} */
{
    /** @constructor */
    initialize: function (a) {
        this.storage = a || jscape.LogService.Type.FILE;
    }
});
jscape.LogService.fromJSON = function(data) {
    return data ? $.extend(new jscape.LogService(), data) : null;
};
jscape.LogService.Type = {FILE: "FILE", DB: "DB"};
jscape.LogService.FILE_STRATEGY = {DAILY: "Daily", WEEKLY: "Weekly", MONTHLY: "Monthly", FILE_SIZE: "FileSize"};
jscape.LogService.file = function(directory, strategy, size) {
    return $.extend(new jscape.LogService(jscape.LogService.Type.FILE), {directory: directory, strategyName: strategy, fileSize: size});
};
jscape.LogService.db = function(url, user, password, poolSize, ttl) {
    return $.extend(new jscape.LogService(jscape.LogService.Type.DB), {url: url, username: user, password: password, poolSize: poolSize, idleConnectionTTL: ttl});
};
jscape.LogService.typeOf = function(descriptor, defaultType) {
    return descriptor != null && $.inArray(descriptor.storage, [jscape.LogService.Type.FILE, jscape.LogService.Type.DB]) != -1
            ? descriptor.storage
            : defaultType || null;
};


/** @class */
jscape.domain.LogServiceController = Class.extend(
    /** @lends {jscape.domain.LogServiceController.prototype} */
{
    DEFAULT_DESCRIPTOR: jscape.LogService.file("", jscape.LogService.FILE_STRATEGY.DAILY, 10 * 1024),

    descriptor: null,
    copy: null,

    /** @constructor */
    initialize: function (resources) {
        this.resources = resources;
        this.view = new jscape.domain.LogServiceView(resources);
        this.controllers = {
            FILE: new jscape.domain.FileLogServiceController(resources),
            DB: new jscape.domain.DbLogServiceController(resources)
        };
    },

    start: function (managementService, domainName) {
        this.managementService = managementService;
        this.domainName = domainName;
        this.view.show(this);
    },

    hasChanged: function() {
        return this.copy && !Object._equals(this.copy, this._createDescriptor());
    },

    onApply: function () {
        var descriptor = this._createDescriptor();
        if (descriptor == null) {
            return $.Deferred().reject().promise();
        }

        return this.managementService.setLogService(this.domainName, descriptor)
                .done($.proxy(function () {
                    this.descriptor = descriptor;
                    this.copy = this._copyDescriptor();
                    this._setupView();
                }, this))
                .done($.proxy(this._fireApplySuccess, this));
    },

    onDiscard: function () {
        this.descriptor = this.copy;
        this.copy = this._copyDescriptor();
        this._setupView();
    },

    onChangeType: function () {
        var serviceType = (this.view.getType() || "").toUpperCase();
        $.each(this.controllers, $.proxy(function(type, controller) {
            if (serviceType === type) {
                controller.resume();
            } else {
                controller.pause();
            }
        }, this));
    },

    _loadData: function () {
        this.managementService.getLogService(this.domainName)
                .done($.proxy(function (descriptor) { this.descriptor = descriptor || this.DEFAULT_DESCRIPTOR; }, this))
                .fail($.proxy(function() { this.descriptor = this.DEFAULT_DESCRIPTOR; }, this))
                .always($.proxy(function() {
                    this.copy = this._copyDescriptor();
                    this._setupView();
                }, this));
    },

    _createDescriptor: function() {
        var controller = this.controllers[(this.view.getType() || "").toUpperCase()];
        return $.type(controller) === "object" ? controller.applyChanges() : null;
    },

    _copyDescriptor: function () {
        return this.descriptor != null ? $.extend(new jscape.LogService(), this.descriptor) : null;
    },

    _setupView: function () {
        for (var type in this.controllers) {
            if (this.controllers.hasOwnProperty(type)) {
                this.controllers[type].start(this.managementService, this.domainName, this.descriptor);
            }
        }
        this.view.setType(jscape.LogService.typeOf(this.descriptor, jscape.LogService.Type.FILE));
    },

    _fireApplySuccess: function () {
        $(document).triggerHandler("broadcast", [this.resources.getString("APPLICATION_SUCCESS_APPLY")]);
    }
});

/** @class */
jscape.domain.LogServiceView = Class.extend(
    /** @lends {jscape.domain.LogServiceView.prototype} */
{
    /** @constructor */
    initialize: function(resources) {
        this.resources = resources;

        this.fieldset = $("fieldset[name=logservicefields]");

        this.type = $("select[name=type]", this.fieldset);
        this.type.combobox({required: true, editable: false, width: 120, panelHeight: "auto", onChange: $.proxy(this._onChange$Type, this)});

        this.applyButton = $("a[role=button][name=apply]", this.fieldset);
        this.applyButton.linkbutton({onClick: $.proxy(this._onClick$ApplyButton, this)});

        this.discardButton = $("a[role=button][name=discard]", this.fieldset);
        this.discardButton.linkbutton({onClick: $.proxy(this._onClick$DiscardButton, this)});
    },

    getType: function() {
        return this.type.combobox("getValue");
    },

    setType: function(value) {
        var select = this.type.combobox("getValue");
        this.type.combobox("setValue", value);
        if (select === value) {
            this._onChange$Type();
        }
    },

    show: function(listener) {
        this.listener = listener || jscape.domain.LogServiceView.LISTENER;
    },

    _onChange$Type: function() {
        this.listener.onChangeType(this);
    },

    _onClick$ApplyButton: function() {
        this.applyButton.linkbutton("disable");
        this.listener.onApply(this)
                .always($.proxy(function() { this.applyButton.linkbutton("enable"); }, this));
    },

    _onClick$DiscardButton: function() {
        this.listener.onDiscard(this);
    }
});
jscape.domain.LogServiceView.LISTENER = {
    onChangeType: $.noop,
    onApply: $.noop,
    onDiscard: $.noop
};

/** @class */
jscape.domain.FileLogServiceController = Class.extend(
    /** @lends {jscape.domain.FileLogServiceController.prototype} */
{
    DEFAULT: {DIRECTORY: "%installdir%/logs/%domain%", STRATEGY: jscape.LogService.FILE_STRATEGY.DAILY},
    VARIABLES: ["installdir", "domain"],

    /** @constructor */
    initialize: function(resources, container) {
        this.view = new jscape.domain.FileLogServiceView(resources, container);
        this.variableController = new jscape.VariableController();
    },

    start: function(managementService, domainName, descriptor) {
        this.managementService = managementService;
        this.domainName = domainName;

        this._setupView(descriptor);
        this.view.show(this);
    },

    resume: function() {
        this.view.show(this);
    },

    pause: function() {
        this.view.hide();
    },

    valid: function() {
        return this.view.valid();
    },

    applyChanges: function() {
        return this.view.valid() ? this._createDescriptor() : null;
    },

    onAddVariable: function(input) {
        this.variableController.start(input, this.VARIABLES);
    },

    onSelectPath: function() {
        var loadFn = $.proxy(this.managementService.listLogDir, this.managementService);
        var createFn = $.proxy(this.managementService.createLogDir , this.managementService);
        new SelectPathController(loadFn, createFn).selectPath($.proxy(function(entry) { this.view.setDirectory(entry.path); }, this));
    },

    _createDescriptor: function() {
        return jscape.LogService.file(this.view.getDirectory(), this.view.getStrategy(), this.view.getFileSize());
    },

    _setupView: function(descriptor) {
        if (jscape.LogService.Type.FILE == jscape.LogService.typeOf(descriptor, null)) {
            this.view.setDirectory(descriptor.directory);
            this.view.setStrategy(descriptor.strategyName);
            this.view.setFileSize(descriptor.fileSize);
        } else {
            this.view.setDirectory(this.DEFAULT.DIRECTORY);
            this.view.setStrategy(this.DEFAULT.STRATEGY);
        }
    }
});

/** @class */
jscape.domain.FileLogServiceView = Class.extend(
    /** @lends {jscape.domain.FileLogServiceView.prototype} */
{
    /** @const */
    SIZE_FACTOR: 1024 * 1024,
    DEFAULT_SIZE: 10,

    /** @constructor */
    initialize: function(resources, container) {
        this.fieldset = $("fieldset[name=filelogservicefields]", container).hide();

        this.directory = $("input[name=directory]", this.fieldset);
        this.directory.validatebox({required: true, validType: "length[1,500]", missingMessage: resources.getString("LOGGING_ERROR_BAD_DIR"), invalidMessage: resources.getString("LOGGING_ERROR_BAD_DIR"), tipPosition: "left"});
        this.directory.focus($.proxy(function() { this.addVariable.linkbutton("enable"); }, this));

        this.strategy = $("input[name=strategy]:radio", this.fieldset);
        this.strategy.change($.proxy(this._adjustFieldState, this));
        this.sizeStrategy = this.strategy.filter("[value=FileSize]");

        this.fileSize = $("input[name=filesize]", this.fieldset);
        this.fileSize.numberspinner({required: true, value: this.DEFAULT_SIZE});

        this.browseButton = $("a[role=button][name=browsedir]", this.fieldset);
        this.browseButton.linkbutton({onClick: $.proxy(this._onClick$BrowseButton, this)});

        this.addVariable = $("a[role=button][name=addvar]", this.fieldset);
        this.addVariable.linkbutton({disabled: true, onClick: $.proxy(this._onClick$AddVariable, this)});
    },

    getDirectory: function() {
        return this.directory.val();
    },

    setDirectory: function(value) {
        this.directory.val(value).validatebox("validate");
    },

    getStrategy: function() {
        return this.strategy.filter(":checked").val();
    },

    setStrategy: function(value) {
        this.strategy.filter("[value=" + value + "]").prop("checked", true).change();
    },

    getFileSize: function() {
        return parseInt(this.fileSize.numberspinner("getValue")) * this.SIZE_FACTOR;
    },

    setFileSize: function(value) {
        this.fileSize.numberspinner("setValue", parseInt(value) / this.SIZE_FACTOR);
    },

    show: function(listener) {
        this.listener = listener || jscape.domain.FileLogServiceView.LISTENER;
        this.fieldset.show();
        this._adjustFieldState();
    },

    hide: function() {
        this.fieldset.hide();
    },

    valid: function() {
        return this.fieldset.form("validate");
    },

    _onClick$AddVariable: function() {
        this.addVariable.linkbutton("disable");
        this.listener.onAddVariable(this.directory);
    },

    _onClick$BrowseButton: function() {
        this.listener.onSelectPath(this);
    },

    _adjustFieldState: function() {
        var enable = this.sizeStrategy.is(":checked");
        this.fileSize.numberspinner(enable ? "enable" : "disable").validatebox(enable ? "enableValidation" : "disableValidation").validatebox("validate");
    }
});
jscape.domain.FileLogServiceView.LISTENER = {
    onAddVariable: $.noop,
    onSelectPath: $.noop
};


/** @class */
jscape.domain.DbLogServiceController = Class.extend(
    /** @lends {jscape.domain.DbLogServiceController.prototype} */
{
    /** @constructor */
    initialize: function(resources, container) {
        this.resources = resources;
        this.view = new jscape.domain.DbLogServiceView(resources, container);
    },

    start: function(managementService, domainName, descriptor) {
        this.managementService = managementService;
        this.domainName = domainName;

        this._setupView(descriptor);
        this.view.show(this);
    },

    resume: function() {
        this.view.show(this);
    },

    pause: function() {
        this.view.hide();
    },

    valid: function() {
        return this.view.valid();
    },

    applyChanges: function () {
        return this.view.valid() ? this._createDescriptor() : null;
    },

    onTest: function() {
        var descriptor = this._createDescriptor();
        this.managementService.testLogService(this.domainName, descriptor)
                .done($.proxy(this._testPassed, this));
    },

    _createDescriptor: function() {
        return jscape.LogService.db(this.view.getUrl(),
                                    this.view.getUsername(),
                                    this.view.getPassword(),
                                    this.view.getPoolSize(),
                                    this.view.getPoolTimeout());
    },

    _setupView: function(descriptor) {
        if (jscape.LogService.Type.DB == jscape.LogService.typeOf(descriptor, null)) {
            this.view.setUrl(descriptor.url);
            this.view.setUsername(descriptor.username);
            this.view.setPassword(descriptor.password);
            this.view.setPoolSize(descriptor.poolSize);
            this.view.setPoolTimeout(descriptor.idleConnectionTTL);
        }
    },

    _testPassed: function () {
        $.messager.alert(this.resources.getString("LOGGING_TEST_DB_TITLE"),
                         this.resources.getString("LOGGING_TEST_DB_PASSED_MESSAGE"),
                         "info");
    }
});

/** @class */
jscape.domain.DbLogServiceView = Class.extend(
    /** @lends {jscape.domain.DbLogServiceView.prototype} */
{
    /** @const */
    TIMEOUT_FACTOR: 60 * 1000,

    /** @constructor */
    initialize: function(resources, container) {
        this.fieldset = $("fieldset[name=dblogservicefields]", container).hide();

        this.url = $("input[name=dburl]", this.fieldset);
        this.url.validatebox({required: true, validType: "length[1,2048]", missingMessage: resources.getString("LOGGING_ERROR_BAD_DB_URL"), invalidMessage: resources.getString("LOGGING_ERROR_BAD_DB_URL")});

        this.username = $("input[name=username]", this.fieldset);
        this.username.validatebox({required: true, validType: "length[1,50]", missingMessage: resources.getString("LOGGING_ERROR_BAD_USER"), invalidMessage: resources.getString("LOGGING_ERROR_BAD_USER")});

        this.password = $("input[name=password]", this.fieldset);

        this.poolSize = $("input[name=poolsize]", this.fieldset);
        this.poolSize.numberspinner({min: 1, max: 100, value: 5, width: 50});

        this.poolTimeout = $("input[name=idlettl]", this.fieldset);
        this.poolTimeout.numberspinner({min: 1, max: 100, value: 5, width: 50});

        this.testButton = $("a[role=button][name=testdb]", this.fieldset);
        this.testButton.linkbutton({onClick: $.proxy(this._onClick$TestButton, this)});
    },

    getUrl: function() {
        return this.url.val();
    },

    setUrl: function(value) {
        this.url.val(value).validatebox("validate");
    },

    getUsername: function() {
        return this.username.val();
    },

    setUsername: function(value) {
        this.username.val(value).validatebox("validate");
    },

    getPassword: function() {
        return this.password.val();
    },

    setPassword: function(value) {
        this.password.val(value);
    },

    getPoolSize: function() {
        return parseInt(this.poolSize.numberspinner("getValue"));
    },

    setPoolSize: function(value) {
        this.poolSize.numberspinner("setValue", parseInt(value));
    },

    getPoolTimeout: function() {
        return parseInt(this.poolTimeout.numberspinner("getValue")) * this.TIMEOUT_FACTOR;
    },

    setPoolTimeout: function(value) {
        this.poolTimeout.numberspinner("setValue", parseInt(value) / this.TIMEOUT_FACTOR);
    },

    show: function(listener) {
        this.listener = listener || jscape.domain.DbLogServiceView.LISTENER;
        this.fieldset.show();
    },

    hide: function() {
        this.fieldset.hide();
    },

    valid: function() {
        return this.fieldset.form("validate");
    },

    _onClick$TestButton: function() {
        if (this.valid()) {
            this.listener.onTest(this);
        }
    }
});
jscape.domain.DbLogServiceView.LISTENER = {
    onTest: $.noop
};


/** @class */
jscape.SyslogService = Class.extend(
    /** @lends {jscape.SyslogService.prototype} */
{
    /** @constructor */
    initialize: function (a, b, c, d) {
        this.host = a || "";
        this.port = parseInt(b) || 0;
        this.facilityCode = parseInt(c) || 1;
        this.processName = d || "";
    }
});
jscape.SyslogService.fromJSON = function (data) {
    return data ? $.extend(new jscape.SyslogService(), data) : null;
};

/** @class */
jscape.LogConfiguration = Class.extend(
    /** @lends {jscape.LogConfiguration.prototype} */
{
    /** @constructor */
    initialize: function(a, b) {
        this.maxBufferRecords = parseInt(a) || 0;
        this.verbose = !!b || false;
    }
});
jscape.LogConfiguration.fromJSON = function(data) {
    return $.extend(new jscape.LogConfiguration(), data);
};

/** @class */
jscape.domain.SyslogController = Class.extend(
    /** @lends {jscape.domain.SyslogController.prototype} */
{
    /** @const */
    DEFAULT_PORT: 514,
    DEFAULT_FACILITY: 1,
    DEFAULT_PROCESS: "JSCAPE",

    descriptor: null,
    copy: null,

    /** @constructor */
    initialize: function (resources) {
        this.resources = resources;
        this.view = new jscape.domain.SyslogView(resources);
    },

    start: function (managementService, domainName) {
        this.managementService = managementService;
        this.domainName = domainName;
        this.view.show(this);
    },

    hasChanged: function() {
        return this.copy && !Object._equals(this.copy, this._createDescriptor());
    },

    onApply: function () {
        if (!this.view.valid()) {
            return $.Deferred().reject().promise();
        }

        var descriptor = this._createDescriptor();
        return this.managementService.setSyslogService(this.domainName, descriptor)
                .done($.proxy(function () {
                    this.descriptor = descriptor;
                    this.copy = this._copyDescriptor();
                    this._setupView();
                }, this))
                .done($.proxy(this._fireApplySuccess, this));
    },

    onDiscard: function () {
        this.descriptor = this.copy;
        this.copy = this._copyDescriptor();
        this._setupView();
    },

    _loadData: function () {
        this.managementService.getSyslogService(this.domainName).done($.proxy(function (descriptor) {
            this.descriptor = descriptor;
            this.copy = this._copyDescriptor();
            this._setupView();
        }, this));
    },

    _createDescriptor: function () {
        if (this.view.isEnabled()) {
            return new jscape.SyslogService(this.view.getHost(),
                                            this.view.getPort(),
                                            this.view.getFacility(),
                                            this.view.getProcessName());
        }
        return null;
    },

    _copyDescriptor: function () {
        return this.descriptor != null ? $.extend(new jscape.SyslogService(), this.descriptor) : null;
    },

    _setupView: function () {
        if (this.descriptor != null) {
            this.view.setEnabled(true);
            this.view.setHost(this.descriptor.host);
            this.view.setPort(this.descriptor.port);
            this.view.setFacility(this.descriptor.facilityCode);
            this.view.setProcessName(this.descriptor.processName);
        } else {
            this.view.setEnabled(false);
            this.view.setHost("");
            this.view.setPort(this.DEFAULT_PORT);
            this.view.setFacility(this.DEFAULT_FACILITY);
            this.view.setProcessName(this.DEFAULT_PROCESS);
        }
    },

    _fireApplySuccess: function () {
        $(document).triggerHandler("broadcast", [this.resources.getString("APPLICATION_SUCCESS_APPLY")]);
    }
});

/** @class */
jscape.domain.SyslogView = Class.extend(
    /** @lends {jscape.domain.SyslogView.prototype} */
{
    /** @constructor */
    initialize: function(resources) {
        this.resources = resources;

        this.fieldset = $("fieldset[name=syslogfields]");

        this.enabled = $("input[name=enabled]:checkbox", this.fieldset);
        this.enabled.change($.proxy(this._adjustFieldState, this));

        this.host = $("input[name=host]", this.fieldset);
        this.host.validatebox({required: true, validType: "length[1,255]", missingMessage: resources.getString("LOGGING_ERROR_BAD_SYSLOG_HOST"), invalidMessage: resources.getString("LOGGING_ERROR_BAD_SYSLOG_HOST")});

        this.port = $("input[name=port]", this.fieldset);
        this.port.numberspinner({required: true, min: 0,  max: 65535, width: 100});

        this.facility = $("select[name=facility]", this.fieldset);
        this.facility.combobox({required: true, editable: false, onLoadSuccess: function(data) { data.length && $(this).combobox("select", data[0].value); }});

        this.process = $("input[name=processname]", this.fieldset);
        this.process.validatebox({required: false, validType: "length[1,32]", invalidMessage: resources.getString("LOGGING_ERROR_BAD_SYSLOG_PROCESS_NAME")});

        this.applyButton = $("a[role=button][name=apply]", this.fieldset);
        this.applyButton.linkbutton({onClick: $.proxy(this._onClick$ApplyButton, this)});

        this.discardButton = $("a[role=button][name=discard]", this.fieldset);
        this.discardButton.linkbutton({onClick: $.proxy(this._onClick$DiscardButton, this)});
    },

    isEnabled: function() {
        return this.enabled.is(":checked");
    },

    setEnabled: function(value) {
        this.enabled.prop("checked", !!value).change();
    },

    getHost: function() {
        return this.host.val();
    },

    setHost: function(value) {
        this.host.val(value);
    },

    getPort: function() {
        return parseInt(this.port.numberspinner("getValue"));
    },

    setPort: function(value) {
        this.port.numberspinner("setValue", parseInt(value));
    },

    getFacility: function() {
        return this.facility.combobox("getValue");
    },

    setFacility: function(value) {
        this.facility.combobox("setValue", value);
    },

    getProcessName: function() {
        return this.process.val();
    },

    setProcessName: function(value) {
        this.process.val(value);
    },

    show: function(listener) {
        this.listener = listener || jscape.domain.SyslogView.LISTENER;
        this._adjustFieldState();
    },

    valid: function() {
        return this.fieldset.form("validate");
    },

    _onClick$ApplyButton: function() {
        this.applyButton.linkbutton("disable");
        this.listener.onApply(this)
                .always($.proxy(function() { this.applyButton.linkbutton("enable"); }, this));
    },

    _onClick$DiscardButton: function() {
        this.listener.onDiscard(this);
    },

    _adjustFieldState: function() {
        var enable = this.isEnabled();
        this.host.prop("disabled", !enable).validatebox(enable ? "enableValidation" : "disableValidation");
        this.port.numberspinner(enable ? "enable" : "disable").validatebox(enable ? "enableValidation" : "disableValidation");
        this.facility.combobox(enable ? "enable" : "disable");
        this.process.prop("disabled", !enable);
        this.fieldset.form("validate");
    }
});
jscape.domain.SyslogView.LISTENER = {
    onApply: $.noop,
    onDiscard: $.noop
};

/** @class */
jscape.domain.LogConfigController = Class.extend(
    /** @lends {jscape.domain.LogConfigController.prototype} */
{
    config: null,
    copy: null,

    /** @constructor */
    initialize: function (resources) {
        this.resources = resources;
        this.view = new jscape.domain.LogConfigView(resources);
    },

    start: function (managementService, domainName) {
        this.managementService = managementService;
        this.domainName = domainName;
        this.view.show(this);
    },

    hasChanged: function() {
        return this.copy && !Object._equals(this.copy, this._createConfiguration());
    },

    onApply: function() {
        if (!this.view.valid()) {
            return $.Deferred().reject().promise();
        }

        var config = this._createConfiguration();
        return this.managementService.setLogConfiguration(this.domainName, config)
                .done($.proxy(function() {
                    this.config = config;
                    this.copy = this._copyConfiguration();
                    this._setupView();
                }, this))
                .done($.proxy(this._fireApplySuccess, this));
    },

    onDiscard: function() {
        this.config = this.copy;
        this.copy = this._copyConfiguration();
        this._setupView();
    },

    _loadData: function() {
        this.managementService.getLogConfiguration(this.domainName).done($.proxy(function(config) {
            this.config = config;
            this.copy = this._copyConfiguration();
            this._setupView();
        }, this));
    },

    _createConfiguration: function() {
        return new jscape.LogConfiguration(this.view.getMaxRecords(), this.view.getVerboseLog());
    },

    _copyConfiguration: function() {
        return $.extend(new jscape.LogConfiguration(), this.config);
    },

    _setupView: function() {
        if (this.config) {
            this.view.setMaxRecords(this.config.maxBufferRecords);
            this.view.setVerboseLog(this.config.verbose);
        }
    },

    _fireApplySuccess: function () {
        $(document).triggerHandler("broadcast", [this.resources.getString("APPLICATION_SUCCESS_APPLY")]);
    }
});

/** @class */
jscape.domain.LogConfigView = Class.extend(
    /** @lends {jscape.domain.LogConfigView.prototype} */
{
    /** @constructor */
    initialize: function(resources) {
        this.resources = resources;

        this.fieldset = $("fieldset[name=logconfigfields]");

        this.maxRecords = $("input[name=buffersize]", this.fieldset);
        this.maxRecords.numberspinner({required: true, min: 10, max: 1000, increment: 10, width: 100});

        this.verbose = $("input[name=verbose]:checkbox", this.fieldset);

        this.applyButton = $("a[role=button][name=apply]", this.fieldset);
        this.applyButton.linkbutton({onClick: $.proxy(this._onClick$ApplyButton, this)});

        this.discardButton = $("a[role=button][name=discard]", this.fieldset);
        this.discardButton.linkbutton({onClick: $.proxy(this._onClick$DiscardButton, this)});
    },

    getMaxRecords: function() {
        return parseInt(this.maxRecords.numberspinner("getValue"));
    },

    setMaxRecords: function(value) {
        this.maxRecords.numberspinner("setValue", parseInt(value)).validatebox("validate");
    },

    getVerboseLog: function() {
        return this.verbose.is(":checked");
    },

    setVerboseLog: function(value) {
        this.verbose.prop("checked", !!value);
    },

    show: function(listener) {
        this.listener = listener || jscape.domain.LogConfigView.LISTENER;
    },

    valid: function() {
        return this.fieldset.form("validate");
    },

    _onClick$ApplyButton: function() {
        this.applyButton.linkbutton("disable");
        this.listener.onApply(this)
                .always($.proxy(function() { this.applyButton.linkbutton("enable"); }, this));
    },

    _onClick$DiscardButton: function() {
        this.listener.onDiscard(this);
    }
});
jscape.domain.LogConfigView.LISTENER = {
    onApply: $.noop,
    onDiscard: $.noop
};