/** @class */
jscape.LogSearch = Class.extend(
    /** @lends {jscape.LogSearch.prototype} */
{
    /** @constructor */
    initialize: function (a, b, c, d, e, f, g, h) {
        this.name = a || "";
        this.maxRecords = parseInt(b) || 0;
        this.date = parseInt(c) || $.now();
        this.startDate = parseInt(d) || null;
        this.endDate = parseInt(e) || null;
        this.criteria = f || "";
        this.progress = parseInt(g) || 0;
        this.error = h || null;
    },

    completed: function() {
        return this.progress == 100 && !this.error;
    }
});
jscape.LogSearch.fromJSON = function(data) {
    return $.extend(new jscape.LogSearch(), data);
};

/** @class */
jscape.SearchCriteria = Class.extend(
    /** @lends {jscape.SearchCriteria.prototype} */
{
    /** @constructor */
    initialize: function(variable, operator, value) {
        this.variable = variable || "";
        this.operator = operator || null;
        this.value = value || null;
    },

    toString: function () {
        return ($.type(this.value) === "string" ? '{0} {1} "{2}"' : "{0} {1} {2}")
                .supplant({0: this.variable, 1: this.operator, 2: this.value});
    }
});
/** @class */
jscape.SearchDateCriteria = Class.extend(
    /** @lends {jscape.SearchDateCriteria.prototype} */
{
    /** @constructor */
    initialize: function(variable, startDate, endDate) {
        this.variable = variable || "";
        this.startDate = startDate || null;
        this.endDate = endDate || null;
    },

    toString: function() {
        return "({0} >= {1} AND {0} <= {2})".supplant({
            0: this.variable,
            1: $.type(this.startDate) === "date" ? this.startDate.getTime() : this.startDate,
            2: $.type(this.endDate) === "date" ? this.endDate.getTime() : this.endDate
        });
    }
});

/** @class */
jscape.domain.LogSearchController = Class.extend(
    /** @lends {jscape.domain.LogSearchController.prototype} */
{
    /** @constructor */
    initialize: function (resources) {
        this.criteriaView = new jscape.domain.LogSearchCriteriaView(resources);
        this.resultsView = new jscape.domain.LogSearchResultsView(resources);
        this.exportController = new jscape.domain.ExportLogSearchController(resources);
        this.resultController = new jscape.domain.LogSearchResultController(resources);
        this.deleteController = new jscape.domain.DeleteLogSearchController(resources);

        this._initValidators(resources);
    },

    start: function (managementService, domainName) {
        this.managementService = managementService;
        this.domainName = domainName;
        this.criteriaView.show(this);
        this.resultsView.show(this);
    },

    onSearch: function() {
        var search = this._createNewSearch();
        this.managementService.searchLog(this.domainName, search)
                .done($.proxy(function() {
                    this._loadData().done($.proxy(function() { this.resultsView.selectItem(search); }, this));
                    this._selectResultsView();
                }, this));
    },

    onReload: function () {
        this._loadData();
    },

    onExport: function (search) {
        this.exportController.start(this.managementService, this.domainName, search.name);
    },

    onView: function (search) {
        this.resultController.start(this.managementService, this.domainName, search.name);
    },

    onRestart: function (search) {
        this.managementService.startLogSearch(this.domainName, search.name)
                .done($.proxy(this._loadData, this));
    },

    onDelete: function (searches) {
        this.deleteController.start(this.managementService, this.domainName, searches)
                .done($.proxy(this._loadData, this));
    },

    _loadData: function () {
        return this.managementService.getLogSearches(this.domainName).done($.proxy(function (searches) {
            this.searches = searches || [];
            this.resultsView.setSearches(this.searches);
        }, this));
    },

    _createNewSearch: function () {
        var dateCriteria = this.criteriaView.getDateCriteria();
        return new jscape.LogSearch(this.criteriaView.getName(),
                                    this.criteriaView.getMaxRecords(),
                                    $.now(),
                                    dateCriteria != null ? dateCriteria.startDate : null,
                                    dateCriteria != null ? dateCriteria.endDate : null,
                                    this._createSearchCriteria());
    },

    _createSearchCriteria: function() {
        return $.map($.grep([
                this.criteriaView.getDateCriteria(),
                this.criteriaView.getClientIpCriteria(),
                this.criteriaView.getClientPortCriteria(),
                this.criteriaView.getServerIpCriteria(),
                this.criteriaView.getServerPortCriteria(),
                this.criteriaView.getUsernameCriteria(),
                this.criteriaView.getClientRequestCriteria(),
                this.criteriaView.getClientMessageCriteria(),
                this.criteriaView.getServerStatusCriteria(),
                this.criteriaView.getServerMessageCriteria(),
                this.criteriaView.getInboundCriteria(),
                this.criteriaView.getOutboundCriteria()
            ], function (criteria) { return criteria != null; }),
                     function (criteria) { return criteria.toString(); }).join(" AND ");
    },

    _selectResultsView: function() {
        this._navigateFrom(this.resultsView.fieldset);
    },

    _navigateFrom: function (source) {
        source.parents(".tabs-container").each($.proxy(function (_, elem) {
            var tabs = $(elem);
            var tabIndex = -1;
            $.each(tabs.tabs("tabs"), function (i, target) {
                if (target.has(source).length != 0) {
                    tabIndex = i;
                    return false;
                }
                return true;
            });
            if (tabIndex != -1) {
                tabs.tabs("select", tabIndex);
            }
            if (this.tabs != tabs) {
                this._navigateFrom(tabs);
            }
        }, this));
    },

    _initValidators: function(resources) {
        var v = new jscape.Validator();
        v.nonEmptyRule("logging_name_nonempty", resources.getString("LOGGING_ERROR_BAD_NAME"));
        v.rule("logging_unique", $.proxy(this._searchNotExists, this), resources.getString("LOGGING_ERROR_BAD_NAME"));
    },

    _searchNotExists: function(value) {
        return $.grep(this.searches, function(search) { return search.name == value; }).length == 0;
    }
});

/** @class */
jscape.domain.LogSearchCriteriaView = Class.extend(
    /** @lends {jscape.domain.LogSearchCriteriaView.prototype} */
{
    /** @constructor */
    initialize: function(resources) {
        this.resources = resources;

        this.fieldset = $("fieldset[name=searchcriteriafields]");

        this.name = $("input[name=name]", this.fieldset);
        this.name.validatebox({required: true, validType: ["logging_name_nonempty", "logging_unique"], missingMessage: resources.getString("LOGGING_ERROR_BAD_NAME")});

        this.maxRecords = $("input[name=maxrecords]", this.fieldset);
        this.maxRecords.numberspinner({required: true, min: 1, max: 100000, value: 1000, increment: 10, width: 125});


        this.dateCriteria = new jscape.domain.SearchCriteriaDateView($("input[name=daterange]:checkbox", this.fieldset),
                                                                     $("input[name=datebegin]", this.fieldset),
                                                                     $("input[name=dateend]", this.fieldset));

        this.clientIpCriteria = new jscape.domain.SearchCriteriaTextView($("input[name=byclientip]:checkbox", this.fieldset),
                                                                         $("select[name=clientipoperator]", this.fieldset),
                                                                         $("input[name=clientip]", this.fieldset));

        this.clientPortCriteria = new jscape.domain.SearchCriteriaPortView($("input[name=byclientport]:checkbox", this.fieldset),
                                                                           $("select[name=clientportoperator]", this.fieldset),
                                                                           $("input[name=clientport]", this.fieldset));

        this.serverIpCriteria = new jscape.domain.SearchCriteriaTextView($("input[name=byserverip]:checkbox", this.fieldset),
                                                                         $("select[name=serveripoperator]", this.fieldset),
                                                                         $("input[name=serverip]", this.fieldset));

        this.serverPortCriteria = new jscape.domain.SearchCriteriaPortView($("input[name=byserverport]:checkbox", this.fieldset),
                                                                           $("select[name=serverportoperator]", this.fieldset),
                                                                           $("input[name=serverport]", this.fieldset));

        this.usernameCriteria = new jscape.domain.SearchCriteriaTextView($("input[name=byuser]:checkbox", this.fieldset),
                                                                         $("select[name=usernameoperator]", this.fieldset),
                                                                         $("input[name=username]", this.fieldset));

        this.clientRequestCriteria = new jscape.domain.SearchCriteriaTextView($("input[name=byclientrequest]:checkbox", this.fieldset),
                                                                          $("select[name=clientrequestoperator]", this.fieldset),
                                                                          $("input[name=clientrequest]", this.fieldset));

        this.clientMessageCriteria = new jscape.domain.SearchCriteriaTextView($("input[name=byclientmessage]:checkbox", this.fieldset),
                                                                          $("select[name=clientmessageoperator]", this.fieldset),
                                                                          $("input[name=clientmessage]", this.fieldset));

        this.serverStatusCriteria = new jscape.domain.SearchCriteriaTextView($("input[name=byserverstatus]:checkbox", this.fieldset),
                                                                             $("select[name=serverstatusoperator]", this.fieldset),
                                                                             $("input[name=serverstatus]", this.fieldset));

        this.serverMessageCriteria = new jscape.domain.SearchCriteriaTextView($("input[name=byservermessage]:checkbox", this.fieldset),
                                                                              $("select[name=servermessageoperator]", this.fieldset),
                                                                              $("input[name=servermessage]", this.fieldset));

        this.inboundCriteria = new jscape.domain.SearchCriteriaSizeView($("input[name=byinbound]:checkbox", this.fieldset),
                                                                        $("select[name=inboundoperator]", this.fieldset),
                                                                        $("input[name=inbound]", this.fieldset),
                                                                        $("select[name=inboundfactor]", this.fieldset));

        this.outboundCriteria = new jscape.domain.SearchCriteriaSizeView($("input[name=byoutbound]:checkbox", this.fieldset),
                                                                         $("select[name=outboundoperator]", this.fieldset),
                                                                         $("input[name=outbound]", this.fieldset),
                                                                         $("select[name=outboundfactor]", this.fieldset));

        this.searchButton = $("a[role=button][name=search]", this.fieldset);
        this.searchButton.linkbutton({onClick: $.proxy(this._onClick$SearchButton, this)});
    },

    getName: function() {
        return this.name.val();
    },

    getMaxRecords: function() {
        return parseInt(this.maxRecords.numberspinner("getValue"));
    },

    getDateCriteria: function() {
        return this.dateCriteria.val();
    },

    getClientIpCriteria: function() {
        return this.clientIpCriteria.val();
    },

    getClientPortCriteria: function() {
        return this.clientPortCriteria.val();
    },

    getServerIpCriteria: function() {
        return this.serverIpCriteria.val();
    },

    getServerPortCriteria: function() {
        return this.serverPortCriteria.val();
    },

    getUsernameCriteria: function() {
        return this.usernameCriteria.val();
    },

    getClientRequestCriteria: function() {
        return this.clientRequestCriteria.val();
    },

    getClientMessageCriteria: function() {
        return this.clientMessageCriteria.val();
    },

    getServerStatusCriteria: function() {
        return this.serverStatusCriteria.val();
    },

    getServerMessageCriteria: function() {
        return this.serverMessageCriteria.val();
    },

    getInboundCriteria: function() {
        return this.inboundCriteria.val();
    },

    getOutboundCriteria: function() {
        return this.outboundCriteria.val();
    },

    show: function(listener) {
        this.listener = listener || jscape.domain.LogSearchCriteriaView.LISTENER;
        this._adjustFieldState();
    },

    _onClick$SearchButton: function() {
        if (this.fieldset.form("validate")) {
            this.listener.onSearch();
        }
    },

    _adjustFieldState: function() {
    }
});
jscape.domain.LogSearchCriteriaView.LISTENER = {
    onSearch: $.noop
};

/** @class */
jscape.domain.SearchCriteriaTextView = Class.extend(
    /** @lends {jscape.domain.SearchCriteriaTextView.prototype} */
{
    /** @constructor */
    initialize: function(enableBox, operatorBox, valueField) {
        this.enabled = enableBox.change($.proxy(function() {
            var enabled = this.enabled.is(":checked");
            this.operator.combobox(enabled ? "enable" : "disable");
            this.value.prop("disabled", !enabled).validatebox(enabled ? "enableValidation" : "disableValidation");
        }, this));
        this.operator = operatorBox.combobox({editable: false, panelHeight: "auto"});
        this.value = valueField.validatebox({required: true});
        this.enabled.change();
    },

    val: function(criteria) {
        if (!arguments.length) {
            return this.enabled.is(":checked")
                    ? new jscape.SearchCriteria(this.enabled.val(), this.operator.combobox("getValue"), this.value.val())
                    : null;
        }
        if (criteria != null && criteria.variable == this.enabled.val(criteria)) {
            this.enabled.prop("checked", true).change();
            this.operator("setValue", criteria.operator);
            this.value.val(criteria.value);
        }
        return this;
    }
});

/** @class */
jscape.domain.SearchCriteriaDateView = Class.extend(
    /** @lends {jscape.domain.SearchCriteriaDateView.prototype} */
{
    /** @constructor */
    initialize: function(enableBox, startDate, endDate) {
        this.enabled = enableBox.change($.proxy(function() {
            var enabled = this.enabled.is(":checked");
            this.startDate.datebox(enabled ? "enable" : "disable").datebox(enabled ? "enableValidation" : "disableValidation");
            this.endDate.datebox(enabled ? "enable" : "disable").datebox(enabled ? "enableValidation" : "disableValidation");
        }, this));
        this.startDate = startDate.datebox({required: true, width: 120});
        this.endDate = endDate.datebox({required: true, width: 100});

        var now = new Date();
        var dateString = (now.getMonth() + 1) + "/" + now.getDate() + "/" + now.getFullYear();
        this.startDate.datebox("setValue", dateString);
        this.endDate.datebox("setValue", dateString);

        this.enabled.change();
    },

    val: function(criteria) {
        if (!arguments.length) {
            return this.enabled.is(":checked")
                    ? new jscape.SearchDateCriteria(this.enabled.val(),
                                                    this.startDate.datebox("options").parser(this.startDate.datebox("getValue")).getTime(),
                                                    this.endDate.datebox("options").parser(this.endDate.datebox("getValue")).getTime())
                    : null;
        }
        if (criteria != null && criteria.variable === this.enabled.val()) {
            this.enabled.prop("checked", true).change();
            this.startDate.datebox("setValue", criteria.startDate);
            this.endDate.datebox("setValue", criteria.endDate);
        }
        return this;
    }
});

/** @class */
jscape.domain.SearchCriteriaPortView = Class.extend(
    /** @lends {jscape.domain.SearchCriteriaPortView.prototype} */
{
    /** @constructor */
    initialize: function(enableBox, operatorBox, portField) {
        this.enabled = enableBox.change($.proxy(function() {
            var enabled = this.enabled.is(":checked");
            this.operator.combobox(enabled ? "enable" : "disable");
            this.port.numberspinner(enabled ? "enable" : "disable").validatebox(enabled ? "enableValidation" : "disableValidation");
        }, this));
        this.operator = operatorBox.combobox({required: true, editable: false, panelHeight: "auto"});
        this.port = portField.numberspinner({required: true, disabled: true, min: 1, max: 65535, value: 1, width: 100});
        this.enabled.change();
    },

    val: function(criteria) {
        if (!arguments.length) {
            return this.enabled.is(":checked")
                    ? new jscape.SearchCriteria(this.enabled.val(), this.operator.combobox("getValue"), parseInt(this.port.numberspinner("getValue")))
                    : null;
        }
        if (criteria != null && criteria.variable === this.enabled.val()) {
            this.enabled.prop("checked", true).change();
            this.operator.combobox("setValue", criteria.variable);
            this.port.numberspinner("setValue", parseInt(criteria.value));
        }
        return this;
    }
});

/** @class */
jscape.domain.SearchCriteriaSizeView = Class.extend(
    /** @lends {jscape.domain.SearchCriteriaSizeView.prototype} */
{
    /** @constructor */
    initialize: function(enableBox, operatorBox, sizeField, factorField) {
        this.enabled = enableBox.change($.proxy(function() {
            var enabled = this.enabled.is(":checked");
            this.operator.combobox(enabled ? "enable" : "disable");
            this.size.numberspinner(enabled ? "enable" : "disable").numberspinner(enabled ? "enableValidation" : "disableValidation");
            this.factor.combobox(enabled ? "enable" : "disable");
        }, this));
        this.operator = operatorBox.combobox({required: true, editable: false, panelHeight: "auto"});
        this.size = sizeField.numberspinner({required: true, disabled: true, min: 1, max: 1000000, value: 1, increment:10, width: 150});
        this.factor = factorField.combobox({editable: false, panelHeight: "auto"});
        this.enabled.change();
    },

    val: function (criteria) {
        if (!arguments.length) {
            return this.enabled.is(":checked")
                    ? new jscape.SearchCriteria(this.enabled.val(),
                                                this.operator.combobox("getValue"),
                                                parseInt(this.size.numberspinner("getValue")) * parseInt(this.factor.combobox("getValue")))
                    : null;
        }
        if (criteria != null && criteria.variable === this.enabled.val()) {
            this.enabled.prop("checked", true).change();
            this.operator.combobox("setValue", criteria.operator);
            var factors = this.factor.combobox("getData");
            var minValue = parseInt(this.size.numberspinner("options").min);
            var maxValue = parseInt(this.size.numberspinner("options").max);
            while (factors) {
                var factor = parseInt(factors.shift());
                var value = parseInt(criteria.value) / factor;
                if (minValue >= value && value <= maxValue) {
                    this.size.numberspinner("setValue", value);
                    this.factor.combobox("select", factor);
                    break;
                }
            }
        }
        return this;
    }
});

/** @class */
jscape.domain.LogSearchResultsView = Class.extend(
    /** @lends {jscape.domain.LogSearchResultsView.prototype} */
{
    selection: null,

    /** @constructor */
    initialize: function(resources) {
        this.fieldset = $("fieldset[name=searchresultsfields]");

        this.data = $("table[role=grid][aria-label=searchresults]", this.fieldset);
        this.data.datagrid({
            fitColumns: true,
            ctrlSelect: true,
            remoteSort: false,
            idField: "name",
            columns: [[
                {field: "name", title: $("thead th:nth-child(1)", this.data).text(), sortable: true, width: 150},
                {field: "date", title: $("thead th:nth-child(2)", this.data).text(), sortable: true, width: 150, align: "right", formatter: function(value) { return new Date(value).toLocaleString(); }},
                {field: "criteria", title: $("thead th:nth-child(3)", this.data).text(), sortable: false, width: 350},
                {field: "status", title: $("thead th:nth-child(4)", this.data).text(), sortable: false, width: 100, formatter: function(_, row) { return row.error ? row.error : row.progress + "%"; }, styler: function(_, row) { return row.error ? "color:red" : "color:green"; }}
            ]],
            onSelect: $.proxy(this._adjustRowSelection, this),
            onUnselect: $.proxy(this._adjustRowSelection, this),
            onLoadSuccess: $.proxy(this._adjustRowSelection, this),
            onDblClickRow: $.proxy(this._onClick$ViewButton, this)
        });

        this.reloadButton = $("a[role=button][name=reload]", this.fieldset);
        this.reloadButton.linkbutton({onClick: $.proxy(this._onClick$Reload, this)});

        this.exportButton = $("a[role=button][name=export]", this.fieldset);
        this.exportButton.linkbutton({disabled: true, onClick: $.proxy(this._onClick$ExportButton, this)});

        this.viewButton = $("a[role=button][name=view]", this.fieldset);
        this.viewButton.linkbutton({disabled: true, onClick: $.proxy(this._onClick$ViewButton, this)});

        this.startButton = $("a[role=button][name=start]", this.fieldset);
        this.startButton.linkbutton({disabled: true, onClick: $.proxy(this._onClick$StartButton, this)});

        this.deleteButton = $("a[role=button][name=delete]", this.fieldset);
        this.deleteButton.linkbutton({disabled: true, onClick: $.proxy(this._onClick$DeleteButton, this)});
    },

    setSearches: function(values) {
        this.data.datagrid("loading");
        this.data.datagrid("clearSelections");
        this.data.datagrid("loadData", values);
        this.data.datagrid("loaded");
    },

    selectItem: function(value) {
        this.data.datagrid("selectRow", this.data.datagrid("getRowIndex", value.name));
    },

    show: function(listener) {
        this.listener = listener || jscape.domain.LogSearchResultsView.LISTENER;
        this.selection = [];
        this._adjustButtonsState();
    },

    _onClick$Reload: function() {
        this.listener.onReload(this);
    },

    _onClick$ExportButton: function() {
        this.listener.onExport(this.selection[0]);
    },

    _onClick$ViewButton: function() {
        this.listener.onView(this.selection[0]);
    },

    _onClick$StartButton: function() {
        this.listener.onRestart(this.selection[0]);
    },

    _onClick$DeleteButton: function() {
        this.listener.onDelete(this.selection);
    },

    _adjustRowSelection: function() {
        this.selection = this.data.datagrid("getSelections");
        this._adjustButtonsState();
    },

    _adjustButtonsState: function () {
        var enabled = this.selection.length == 1 && this._complete(this.selection);
        this.exportButton.linkbutton(enabled ? "enable" : "disable");
        this.viewButton.linkbutton(enabled ? "enable" : "disable");
        this.startButton.linkbutton(enabled ? "enable" : "disable");
        this.deleteButton.linkbutton(this.selection.length != 0 && this._complete(this.selection) ? "enable" : "disable");
    },

    _complete: function(searches) {
        return $.grep(searches || [], function(search) { return search.progress != 100 || search.error; }).length == 0;
    }
});
jscape.domain.LogSearchResultsView.LISTENER = {
    onReload: $.noop,
    onExport: $.noop,
    onView: $.noop,
    onRestart: $.noop,
    onDelete: $.noop
};

/** @class */
jscape.domain.LogSearchResultController = Class.extend(
    /** @lends {jscape.domain.LogSearchResultController.prototype} */
{
    result: null,

    /** @constructor */
    initialize: function(resources) {
        this.resources = resources;
        this.exportController = new jscape.domain.ExportLogSearchController(resources);
    },

    start: function(managementService, domainName, searchName) {
        this.managementService = managementService;
        this.domainName = domainName;
        this.searchName = searchName;

        this.managementService.getLogSearchResult(this.domainName, searchName)
                .done($.proxy(function(result) { this.result = result; }, this))
                .always($.proxy(this._setupDialog, this));
    },

    onExport: function() {
        this.exportController.start(this.managementService, this.domainName, this.searchName);
    },

    onClose: function(dialog) {
        dialog.destroy();
    },

    _setupDialog: function() {
        var dialog = new jscape.domain.LogSearchResultDialog(this.resources);
        dialog.setData(this.result);
        dialog.show(this);
    }
});

/** @class */
jscape.domain.LogSearchResultDialog = Class.extend(
    /** @lends {jscape.domain.LogSearchResultDialog.prototype} */
{
    /** @constructor */
    initialize: function(resources) {
        this.dialog = $("#d-logging-view").clone().attr("id", "");
        this.dialog.dialog({
            width: 800, height: "auto",
            modal: true, resizable: true, closable: true, closed: true,
            buttons: [
                {text: resources.getString("DIALOG_BUTTON_CLOSE") || "Cancel", handler: $.proxy(this._onCancel, this)}
            ],
            onClose: $.proxy(function () {if (!this.closing) { this._onCancel(); }}, this)
        }).show();

        this.fieldset = $("fieldset[name=logviewfields]", this.dialog);

        this.text = $("textarea[name=result]", this.fieldset);
        this.text.prop("readonly", true).css({width: "52em", height: "28em"});

        this.exportButton = $("a[role=button][name=export]", this.fieldset);
        this.exportButton.linkbutton({disabled: true, onClick: $.proxy(this._onClick$ExportButton, this)});
    },

    setData: function(value) {
        this.text.val(value);
        this.exportButton.linkbutton(value && value.trim().length != 0 ? "enable" : "disable");
    },

    show: function (listener) {
        this.listener = listener || jscape.domain.LogSearchResultDialog.LISTENER;
        this.dialog.dialog("center").dialog("open");
    },

    hide: function () {
        if (!this._closed()) {
            this.closing = true;
            this.dialog.dialog("close");
            this.closing = false;
        }
    },

    destroy: function() {
        this.dialog.dialog("destroy");
    },

    _onClick$ExportButton: function() {
        this.listener.onExport(this);
    },

    _onCancel: function () {
        this.hide();
        this.listener.onClose(this);
    },

    _closed: function () {
        return !!this.dialog.dialog("options").closed;
    }
});
jscape.domain.LogSearchResultDialog.LISTENER = {
    onExport: $.noop,
    onClose: $.noop
};

/** @class */
jscape.domain.ExportLogSearchController = Class.extend(
    /** @lends {jscape.domain.ExportLogSearchController.prototype} */
{
    deferred: null,

    /** @constructor */
    initialize: function(resources) {
        this.resources = resources;
    },

    start: function(managementService, domainName, searchName) {
        this.managementService = managementService;
        this.domainName = domainName;
        this.searchName = searchName;
        this.deferred = $.Deferred();

        var dialog = new jscape.domain.ExportLogSearchView(this.resources);
        dialog.show(this);

        return this.deferred.promise();
    },

    onSubmit: function(dialog) {
        var filename = dialog.getFilename();
        dialog.destroy();
        this.managementService.exportLogSearchResult(this.domainName, this.searchName, filename)
                .done(this.deferred.resolve)
                .fail(this.deferred.reject);
    },

    onCancel: function(dialog) {
        dialog.destroy();
        this.deferred.reject();
    }
});

/** @class */
jscape.domain.ExportLogSearchView = Class.extend(
    /** @lends {jscape.domain.ExportLogSearchView.prototype} */
{
    /** @constructor */
    initialize: function(resources) {
        this.dialog = $("#d-logging-export").clone().attr("id", "");
        this.dialog.dialog({
            width: 530, height: "auto",
            modal: true, resizable: false, closable: true, closed: true,
            buttons: [
                {text: resources.getString("DIALOG_BUTTON_OK") || "Ok", handler: $.proxy(this._onSubmit, this)},
                {text: resources.getString("DIALOG_BUTTON_CANCEL") || "Cancel", handler: $.proxy(this._onCancel, this)}
            ],
            onClose: $.proxy(function () {if (!this.closing) { this._onCancel(); }}, this)
        }).show();

        this.fieldset = $("fieldset[name=logexportfields]", this.dialog);
        this.filename = $("input[name=filename]", this.fieldset);
        this.filename.validatebox({required: true});
    },

    getFilename: function() {
        return this.filename.val();
    },

    show: function (listener) {
        this.listener = listener;
        this.dialog.dialog("center").dialog("open");
        this.fieldset.form("validate");
        this.filename.focus();
    },

    hide: function () {
        if (!this._closed()) {
            this.closing = true;
            this.dialog.dialog("close");
            this.fieldset.form("disableValidation");
            this.closing = false;
        }
    },

    destroy: function() {
        this.dialog.dialog("destroy");
    },

    _onSubmit: function () {
        if (this.fieldset.form("validate")) {
            this.hide();
            this.listener.onSubmit(this);
        }
    },

    _onCancel: function () {
        this.hide();
        this.listener.onCancel(this);
    },

    _closed: function () {
        return !!this.dialog.dialog("options").closed;
    }
});

/** @class */
jscape.domain.DeleteLogSearchController = Class.extend(
    /** @lends {jscape.domain.DeleteLogSearchController.prototype} */
{
    deferred: null,

    /** @constructor */
    initialize: function(resources) {
        this.resources = resources;
    },

    start: function(managementService, domainName, searches) {
        this.managementService = managementService;
        this.domainName = domainName;
        this.deferred = $.Deferred();

        this._confirmDelete(searches)
                .done($.proxy(this._deleteConfirmed, this, searches))
                .fail(this.deferred.reject);

        return this.deferred.promise();
    },

    _deleteConfirmed: function (searches) {
        var promises = new Array(searches.length);
        for (var i = 0; i < searches.length; i++) {
            promises[i] = this.managementService.deleteLogSearch(this.domainName, searches[i].name);
        }

        $.when.apply(jQuery, promises)
                .done($.proxy(function () { this.deferred.resolve(searches); }, this))
                .fail($.proxy(function() { this.deferred.reject(); }, this));
    },

    _confirmDelete: function(searches) {
        var text = $.map(searches, function(search) { return search.name; }).join(", ");
        return jscape.ConfirmDialog.confirm(this.resources.getString("LOGGING_CONFIRM_DELETE_TITLE"),
                                            this.resources.getString("LOGGING_CONFIRM_DELETE_MESSAGE", {0: text}));
    }
});